/*
 *  udbcodb3.h
 *
 *  $Id: udbcodb3.h,v 1.2.2.1 2001/09/20 11:56:22 source Exp $
 *
 *  ODBC header information
 *
 *  (C)Copyright 1999 OpenLink Software.
 *  All Rights Reserved.
 *
 *  The copyright above and this notice must be preserved in all
 *  copies of this source code.  The copyright above does not
 *  evidence any actual or intended publication of this source code.
 *
 *  This is unpublished proprietary trade secret of OpenLink Software.
 *  This source code may not be copied, disclosed, distributed, demonstrated
 *  or licensed except as authorized by OpenLink Software.
 */

#if (ODBCVER < 0x0300)
#define __UDBCODB3_H		/* These defines are 3.0 and higher ONLY */
#endif

#ifndef __UDBCODB3_H
#define __UDBCODB3_H

#ifndef __UDBCODBC_H
#include <udbc/udbcodbc.h>
#endif

typedef SDWORD               SQLHDESC;
typedef void FAR *	     SQLHANDLE;

/* date/time length constants */
#define SQL_DATE_LEN           10
#define SQL_TIME_LEN            8  /* add P+1 if precision is nonzero */
#define SQL_TIMESTAMP_LEN      19  /* add P+1 if precision is nonzero */

/* handle type identifiers */
#define SQL_HANDLE_ENV             		1
#define SQL_HANDLE_DBC             		2
#define SQL_HANDLE_STMT            		3
#define SQL_HANDLE_DESC            		4

/* environment attribute */
#define SQL_ATTR_OUTPUT_NTS    			10001

/* connection attributes */
#define SQL_ATTR_AUTO_IPD      			10001
#define SQL_ATTR_METADATA_ID   			10014

/* statement attributes */
#define SQL_ATTR_APP_ROW_DESC       		10010
#define SQL_ATTR_APP_PARAM_DESC     		10011
#define SQL_ATTR_IMP_ROW_DESC       		10012
#define SQL_ATTR_IMP_PARAM_DESC     		10013
#define SQL_ATTR_CURSOR_SCROLLABLE  		(-1)
#define SQL_ATTR_CURSOR_SENSITIVITY 		(-2)

/* SQL_ATTR_CURSOR_SCROLLABLE values */
#define SQL_NONSCROLLABLE			0
#define SQL_SCROLLABLE				1

/* identifiers of fields in the SQL descriptor */
#define SQL_DESC_COUNT                  	1001
#define SQL_DESC_TYPE                   	1002
#define SQL_DESC_LENGTH                 	1003
#define SQL_DESC_OCTET_LENGTH_PTR       	1004
#define SQL_DESC_PRECISION              	1005
#define SQL_DESC_SCALE                  	1006
#define SQL_DESC_DATETIME_INTERVAL_CODE 	1007
#define SQL_DESC_NULLABLE               	1008
#define SQL_DESC_INDICATOR_PTR          	1009
#define SQL_DESC_DATA_PTR               	1010
#define SQL_DESC_NAME                   	1011
#define SQL_DESC_UNNAMED                	1012
#define SQL_DESC_OCTET_LENGTH           	1013
#define SQL_DESC_ALLOC_TYPE             	1099

/* identifiers of fields in the diagnostics area */
#define SQL_DIAG_RETURNCODE        		1
#define SQL_DIAG_NUMBER            		2
#define SQL_DIAG_ROW_COUNT         		3
#define SQL_DIAG_SQLSTATE          		4
#define SQL_DIAG_NATIVE            		5
#define SQL_DIAG_MESSAGE_TEXT      		6
#define SQL_DIAG_DYNAMIC_FUNCTION  		7
#define SQL_DIAG_CLASS_ORIGIN      		8
#define SQL_DIAG_SUBCLASS_ORIGIN   		9
#define SQL_DIAG_CONNECTION_NAME  		10
#define SQL_DIAG_SERVER_NAME      		11
#define SQL_DIAG_DYNAMIC_FUNCTION_CODE 		12

/* dynamic function codes */
#define SQL_DIAG_ALTER_TABLE            	4
#define SQL_DIAG_CREATE_INDEX          		(-1)
#define SQL_DIAG_CREATE_TABLE          		77
#define SQL_DIAG_CREATE_VIEW           		84
#define SQL_DIAG_DELETE_WHERE          		19
#define SQL_DIAG_DROP_INDEX            		(-2)
#define SQL_DIAG_DROP_TABLE            		32
#define SQL_DIAG_DROP_VIEW             		36
#define SQL_DIAG_DYNAMIC_DELETE_CURSOR 		38
#define SQL_DIAG_DYNAMIC_UPDATE_CURSOR 		81
#define SQL_DIAG_GRANT                 		48
#define SQL_DIAG_INSERT                		50
#define SQL_DIAG_REVOKE                		59
#define SQL_DIAG_SELECT_CURSOR         		85
#define SQL_DIAG_UNKNOWN_STATEMENT     		 0
#define SQL_DIAG_UPDATE_WHERE          		82

/* SQL data type codes */
#define	SQL_UNKNOWN_TYPE			0
#define SQL_DATETIME        			9

/* One-parameter shortcuts for date/time data types */
#define SQL_TYPE_DATE      			91
#define SQL_TYPE_TIME      			92
#define SQL_TYPE_TIMESTAMP 			93

/* Statement attribute values for cursor sensitivity */
#define SQL_UNSPECIFIED     			0
#define SQL_INSENSITIVE     			1
#define SQL_SENSITIVE       			2

/* Default conversion code for SQLBindCol(), SQLBindParam() and SQLGetData() */
#define SQL_DEFAULT        			99

/* SQLGetData() code indicating that the application row descriptor
 * specifies the data type
 */
#define SQL_ARD_TYPE      			(-99)

/* SQL date/time type subcodes */
#define SQL_CODE_DATE       			1
#define SQL_CODE_TIME       			2
#define SQL_CODE_TIMESTAMP  			3

/* CLI option values */
#define SQL_FALSE           			0
#define SQL_TRUE            			1

/* Values returned by SQLGetTypeInfo() to show WHERE clause
 * supported
 */
#define SQL_PRED_NONE     			0
#define SQL_PRED_CHAR     			1
#define SQL_PRED_BASIC    			2

/* values of UNNAMED field in descriptor */
#define SQL_NAMED           			0
#define SQL_UNNAMED         			1

/* values of ALLOC_TYPE field in descriptor */
#define SQL_DESC_ALLOC_AUTO 			1
#define SQL_DESC_ALLOC_USER 			2

/* null handles returned by SQLAllocHandle() */
#define SQL_NULL_HDESC      			0

/* null handle used in place of parent handle when allocating HENV */
#define SQL_NULL_HANDLE     			0L

/* Reserved value for the IdentifierType argument of SQLSpecialColumns() */
#define SQL_ROW_IDENTIFIER  			1

/* SQLGetFunctions() values to identify ODBC APIs */
#define SQL_API_SQLALLOCHANDLE       		1001
#define SQL_API_SQLBINDPARAM         		1002
#define SQL_API_SQLCLOSECURSOR       		1003
#define SQL_API_SQLCOPYDESC          		1004
#define SQL_API_SQLENDTRAN           		1005
#define SQL_API_SQLFETCHSCROLL       		1021
#define SQL_API_SQLFREEHANDLE        		1006
#define SQL_API_SQLGETCONNECTATTR    		1007
#define SQL_API_SQLGETDESCFIELD      		1008
#define SQL_API_SQLGETDESCREC        		1009
#define SQL_API_SQLGETDIAGFIELD      		1010
#define SQL_API_SQLGETDIAGREC        		1011
#define SQL_API_SQLGETENVATTR        		1012
#define SQL_API_SQLGETSTMTATTR       		1014
#define SQL_API_SQLSETCONNECTATTR    		1016
#define SQL_API_SQLSETDESCFIELD      		1017
#define SQL_API_SQLSETDESCREC        		1018
#define SQL_API_SQLSETENVATTR        		1019
#define SQL_API_SQLSETSTMTATTR       		1020

/* Information requested by SQLGetInfo() */
#define SQL_MAX_DRIVER_CONNECTIONS           	0
#define SQL_MAXIMUM_DRIVER_CONNECTIONS		SQL_MAX_DRIVER_CONNECTIONS
#define SQL_MAX_CONCURRENT_ACTIVITIES        	1
#define SQL_MAXIMUM_CONCURRENT_ACTIVITIES	SQL_MAX_CONCURRENT_ACTIVITIES
#define SQL_MAXIMUM_COLUMN_NAME_LENGTH		SQL_MAX_COLUMN_NAME_LEN
#define SQL_MAXIMUM_CURSOR_NAME_LENGTH		SQL_MAX_CURSOR_NAME_LEN
#define SQL_MAXIMUM_SCHEMA_NAME_LENGTH		SQL_MAX_SCHEMA_NAME_LEN
#define SQL_MAXIMUM_CATALOG_NAME_LENGTH		SQL_MAX_CATALOG_NAME_LEN
#define SQL_TRANSACTION_CAPABLE			SQL_TXN_CAPABLE
#define SQL_TRANSACTION_ISOLATION_OPTION	SQL_TXN_ISOLATION_OPTION
#define SQL_MAXIMUM_COLUMNS_IN_GROUP_BY		SQL_MAX_COLUMNS_IN_GROUP_BY
#define SQL_MAXIMUM_COLUMNS_IN_INDEX		SQL_MAX_COLUMNS_IN_INDEX
#define SQL_MAXIMUM_COLUMNS_IN_ORDER_BY		SQL_MAX_COLUMNS_IN_ORDER_BY
#define SQL_MAXIMUM_COLUMNS_IN_SELECT	   	SQL_MAX_COLUMNS_IN_SELECT
#define SQL_MAXIMUM_INDEX_SIZE		   	SQL_MAX_INDEX_SIZE
#define SQL_MAXIMUM_ROW_SIZE			SQL_MAX_ROW_SIZE
#define SQL_MAXIMUM_STATEMENT_LENGTH	   	SQL_MAX_STATEMENT_LEN
#define SQL_MAXIMUM_TABLES_IN_SELECT	   	SQL_MAX_TABLES_IN_SELECT
#define SQL_MAXIMUM_USER_NAME_LENGTH	   	SQL_MAX_USER_NAME_LEN

/* env attribute */
#define SQL_ATTR_ODBC_VERSION			200
#define SQL_ATTR_CONNECTION_POOLING		201
#define SQL_ATTR_CP_MATCH			202

/* values for SQL_ATTR_CONNECTION_POOLING */
#define SQL_CP_OFF				0UL
#define SQL_CP_ONE_PER_DRIVER			1UL
#define SQL_CP_ONE_PER_HENV			2UL
#define SQL_CP_DEFAULT				SQL_CP_OFF

/* values for SQL_ATTR_CP_MATCH */
#define SQL_CP_STRICT_MATCH			0UL
#define SQL_CP_RELAXED_MATCH			1UL
#define SQL_CP_MATCH_DEFAULT			SQL_CP_STRICT_MATCH		

/* values for SQL_ATTR_ODBC_VERSION */
#define SQL_OV_ODBC2				2UL
#define	SQL_OV_ODBC3				3UL


/* connection attributes with new names */
#define SQL_ATTR_ACCESS_MODE			SQL_ACCESS_MODE
#define SQL_ATTR_AUTOCOMMIT			SQL_AUTOCOMMIT
#if (ODBCVER >= 0x0350)
#define SQL_ATTR_CONNECTION_DEAD		1209/* GetConnectAttr only */
#endif /* ODBCVER >= 0x0350 */
#define SQL_ATTR_CONNECTION_TIMEOUT		113
#define SQL_ATTR_CURRENT_CATALOG		SQL_CURRENT_QUALIFIER
#define SQL_ATTR_DISCONNECT_BEHAVIOR		114
#define SQL_ATTR_ENLIST_IN_DTC			1207
#define SQL_ATTR_ENLIST_IN_XA			1208
#define SQL_ATTR_LOGIN_TIMEOUT			SQL_LOGIN_TIMEOUT
#define SQL_ATTR_ODBC_CURSORS			SQL_ODBC_CURSORS
#define SQL_ATTR_PACKET_SIZE			SQL_PACKET_SIZE
#define SQL_ATTR_QUIET_MODE			SQL_QUIET_MODE
#define SQL_ATTR_TRACE				SQL_OPT_TRACE
#define SQL_ATTR_TRACEFILE			SQL_OPT_TRACEFILE
#define SQL_ATTR_TRANSLATE_LIB			SQL_TRANSLATE_DLL
#define SQL_ATTR_TRANSLATE_OPTION		SQL_TRANSLATE_OPTION
#define SQL_ATTR_TXN_ISOLATION			SQL_TXN_ISOLATION

/* values for SQL_ATTR_DISCONNECT_BEHAVIOR */
#define SQL_DB_RETURN_TO_POOL			0UL
#define SQL_DB_DISCONNECT			1UL
#define SQL_DB_DEFAULT				SQL_DB_RETURN_TO_POOL

/* values for SQL_ATTR_ENLIST_IN_DTC */
#define SQL_DTC_DONE				0L

#if (ODBCVER >= 0x0350)
/* values for SQL_ATTR_CONNECTION_DEAD */
#define SQL_CD_TRUE				1L 
#define SQL_CD_FALSE				0L
#endif /* ODBCVER >= 0x0350 */

/* statement attributes for ODBC 3.0 */
#define SQL_ATTR_ASYNC_ENABLE			4
#define SQL_ATTR_CONCURRENCY			SQL_CONCURRENCY
#define SQL_ATTR_CURSOR_TYPE			SQL_CURSOR_TYPE
#define	SQL_ATTR_ENABLE_AUTO_IPD		15
#define SQL_ATTR_FETCH_BOOKMARK_PTR		16
#define SQL_ATTR_KEYSET_SIZE			SQL_KEYSET_SIZE
#define SQL_ATTR_MAX_LENGTH			SQL_MAX_LENGTH
#define SQL_ATTR_MAX_ROWS			SQL_MAX_ROWS
#define SQL_ATTR_NOSCAN				SQL_NOSCAN
#define SQL_ATTR_PARAM_BIND_OFFSET_PTR		17
#define	SQL_ATTR_PARAM_BIND_TYPE		18
#define SQL_ATTR_PARAM_OPERATION_PTR		19
#define SQL_ATTR_PARAM_STATUS_PTR		20
#define	SQL_ATTR_PARAMS_PROCESSED_PTR		21
#define	SQL_ATTR_PARAMSET_SIZE			22
#define SQL_ATTR_QUERY_TIMEOUT			SQL_QUERY_TIMEOUT
#define SQL_ATTR_RETRIEVE_DATA			SQL_RETRIEVE_DATA
#define SQL_ATTR_ROW_BIND_OFFSET_PTR		23
#define	SQL_ATTR_ROW_BIND_TYPE			SQL_BIND_TYPE
#define SQL_ATTR_ROW_NUMBER			SQL_ROW_NUMBER	/*GetStmtAttr*/
#define SQL_ATTR_ROW_OPERATION_PTR		24
#define	SQL_ATTR_ROW_STATUS_PTR			25
#define	SQL_ATTR_ROWS_FETCHED_PTR		26
#define SQL_ATTR_ROW_ARRAY_SIZE			27	
#define SQL_ATTR_SIMULATE_CURSOR		SQL_SIMULATE_CURSOR
#define SQL_ATTR_USE_BOOKMARKS			SQL_USE_BOOKMARKS	

#if (ODBCVER < 0x0300)
#define SQL_STMT_OPT_MAX               		SQL_ROW_NUMBER
#define SQL_STMT_OPT_MIN			SQL_QUERY_TIMEOUT
#endif 	/* ODBCVER < 0x0300 */

/* New defines for SEARCHABLE column in SQLGetTypeInfo */
#define	SQL_COL_PRED_CHAR			SQL_LIKE_ONLY
#define	SQL_COL_PRED_BASIC			SQL_ALL_EXCEPT_LIKE


/* whether an attribute is a pointer or not */
#define SQL_IS_POINTER				(-4)
#define SQL_IS_UINTEGER				(-5)
#define SQL_IS_INTEGER				(-6)
#define SQL_IS_USMALLINT			(-7)
#define SQL_IS_SMALLINT				(-8)

/* the value of SQL_ATTR_PARAM_BIND_TYPE */
#define SQL_PARAM_BIND_BY_COLUMN		0UL
#define SQL_PARAM_BIND_TYPE_DEFAULT		SQL_PARAM_BIND_BY_COLUMN


/* New values for SQL_USE_BOOKMARKS attribute */
#define SQL_UB_FIXED				SQL_UB_ON
#define SQL_UB_VARIABLE				2UL

/* extended descriptor field */
#define SQL_DESC_ARRAY_SIZE			20
#define SQL_DESC_ARRAY_STATUS_PTR		21
#define SQL_DESC_AUTO_UNIQUE_VALUE		SQL_COLUMN_AUTO_INCREMENT
#define SQL_DESC_BASE_COLUMN_NAME		22
#define SQL_DESC_BASE_TABLE_NAME		23
#define SQL_DESC_BIND_OFFSET_PTR		24
#define SQL_DESC_BIND_TYPE			25
#define SQL_DESC_CASE_SENSITIVE			SQL_COLUMN_CASE_SENSITIVE
#define SQL_DESC_CATALOG_NAME			SQL_COLUMN_QUALIFIER_NAME
#define SQL_DESC_CONCISE_TYPE			SQL_COLUMN_TYPE
#define SQL_DESC_DATETIME_INTERVAL_PRECISION	26
#define SQL_DESC_DISPLAY_SIZE			SQL_COLUMN_DISPLAY_SIZE
#define SQL_DESC_FIXED_PREC_SCALE		SQL_COLUMN_MONEY
#define SQL_DESC_LABEL				SQL_COLUMN_LABEL
#define SQL_DESC_LITERAL_PREFIX			27
#define SQL_DESC_LITERAL_SUFFIX			28
#define SQL_DESC_LOCAL_TYPE_NAME		29
#define	SQL_DESC_MAXIMUM_SCALE			30
#define SQL_DESC_MINIMUM_SCALE			31
#define SQL_DESC_NUM_PREC_RADIX			32
#define SQL_DESC_PARAMETER_TYPE			33
#define SQL_DESC_ROWS_PROCESSED_PTR		34
#if (ODBCVER >= 0x0350)
#define SQL_DESC_ROWVER				35
#endif /* ODBCVER >= 0x0350 */
#define SQL_DESC_SCHEMA_NAME			SQL_COLUMN_OWNER_NAME
#define SQL_DESC_SEARCHABLE			SQL_COLUMN_SEARCHABLE
#define SQL_DESC_TYPE_NAME			SQL_COLUMN_TYPE_NAME
#define SQL_DESC_TABLE_NAME			SQL_COLUMN_TABLE_NAME
#define SQL_DESC_UNSIGNED			SQL_COLUMN_UNSIGNED
#define SQL_DESC_UPDATABLE			SQL_COLUMN_UPDATABLE

/* defines for diagnostics fields */
#define SQL_DIAG_CURSOR_ROW_COUNT		(-1249)
#define SQL_DIAG_ROW_NUMBER			(-1248)
#define SQL_DIAG_COLUMN_NUMBER			(-1247)	

/* dynamic function codes */
#define SQL_DIAG_CALL				7

/* SQL extended datatypes */
#define SQL_INTERVAL				10

#if (ODBCVER >= 0x0300)
/* interval code */
#define SQL_CODE_YEAR				1
#define SQL_CODE_MONTH				2
#define SQL_CODE_DAY				3
#define SQL_CODE_HOUR				4
#define SQL_CODE_MINUTE				5
#define SQL_CODE_SECOND				6
#define SQL_CODE_YEAR_TO_MONTH			7
#define SQL_CODE_DAY_TO_HOUR			8
#define SQL_CODE_DAY_TO_MINUTE			9
#define SQL_CODE_DAY_TO_SECOND			10
#define SQL_CODE_HOUR_TO_MINUTE			11
#define SQL_CODE_HOUR_TO_SECOND			12
#define SQL_CODE_MINUTE_TO_SECOND		13

#define SQL_INTERVAL_YEAR			(100 + SQL_CODE_YEAR)
#define SQL_INTERVAL_MONTH			(100 + SQL_CODE_MONTH)
#define SQL_INTERVAL_DAY			(100 + SQL_CODE_DAY) 
#define SQL_INTERVAL_HOUR			(100 + SQL_CODE_HOUR) 
#define SQL_INTERVAL_MINUTE			(100 + SQL_CODE_MINUTE) 
#define SQL_INTERVAL_SECOND            	    	(100 + SQL_CODE_SECOND) 
#define SQL_INTERVAL_YEAR_TO_MONTH		(100 + SQL_CODE_YEAR_TO_MONTH)
#define SQL_INTERVAL_DAY_TO_HOUR		(100 + SQL_CODE_DAY_TO_HOUR) 
#define SQL_INTERVAL_DAY_TO_MINUTE		(100 + SQL_CODE_DAY_TO_MINUTE) 
#define SQL_INTERVAL_DAY_TO_SECOND		(100 + SQL_CODE_DAY_TO_SECOND) 
#define SQL_INTERVAL_HOUR_TO_MINUTE		(100 + SQL_CODE_HOUR_TO_MINUTE)
#define SQL_INTERVAL_HOUR_TO_SECOND		(100 + SQL_CODE_HOUR_TO_SECOND) 
#define SQL_INTERVAL_MINUTE_TO_SECOND		(100 + SQL_CODE_MINUTE_TO_SECOND) 

#else
#define SQL_INTERVAL_YEAR                       (-80)
#define SQL_INTERVAL_MONTH                      (-81)
#define SQL_INTERVAL_YEAR_TO_MONTH              (-82)
#define SQL_INTERVAL_DAY                        (-83)
#define SQL_INTERVAL_HOUR                       (-84)
#define SQL_INTERVAL_MINUTE                     (-85)
#define SQL_INTERVAL_SECOND                     (-86)
#define SQL_INTERVAL_DAY_TO_HOUR                (-87)
#define SQL_INTERVAL_DAY_TO_MINUTE              (-88)
#define SQL_INTERVAL_DAY_TO_SECOND              (-89)
#define SQL_INTERVAL_HOUR_TO_MINUTE             (-90)
#define SQL_INTERVAL_HOUR_TO_SECOND             (-91)
#define SQL_INTERVAL_MINUTE_TO_SECOND           (-92)
#endif  /* ODBCVER >= 0x0300 */


#define SQL_UNICODE                             (-95)
#define SQL_UNICODE_VARCHAR                     (-96)
#define SQL_UNICODE_LONGVARCHAR                 (-97)
#define SQL_UNICODE_CHAR                        SQL_UNICODE

#define	SQL_C_NUMERIC				SQL_NUMERIC

#define SQL_C_TYPE_DATE				SQL_TYPE_DATE
#define SQL_C_TYPE_TIME				SQL_TYPE_TIME
#define SQL_C_TYPE_TIMESTAMP			SQL_TYPE_TIMESTAMP
#define SQL_C_INTERVAL_YEAR			SQL_INTERVAL_YEAR
#define SQL_C_INTERVAL_MONTH			SQL_INTERVAL_MONTH
#define SQL_C_INTERVAL_DAY			SQL_INTERVAL_DAY
#define SQL_C_INTERVAL_HOUR			SQL_INTERVAL_HOUR
#define SQL_C_INTERVAL_MINUTE			SQL_INTERVAL_MINUTE
#define SQL_C_INTERVAL_SECOND			SQL_INTERVAL_SECOND
#define SQL_C_INTERVAL_YEAR_TO_MONTH		SQL_INTERVAL_YEAR_TO_MONTH
#define SQL_C_INTERVAL_DAY_TO_HOUR		SQL_INTERVAL_DAY_TO_HOUR
#define SQL_C_INTERVAL_DAY_TO_MINUTE		SQL_INTERVAL_DAY_TO_MINUTE
#define SQL_C_INTERVAL_DAY_TO_SECOND		SQL_INTERVAL_DAY_TO_SECOND
#define SQL_C_INTERVAL_HOUR_TO_MINUTE		SQL_INTERVAL_HOUR_TO_MINUTE
#define SQL_C_INTERVAL_HOUR_TO_SECOND		SQL_INTERVAL_HOUR_TO_SECOND
#define SQL_C_INTERVAL_MINUTE_TO_SECOND		SQL_INTERVAL_MINUTE_TO_SECOND
#define SQL_C_SBIGINT	(SQL_BIGINT+SQL_SIGNED_OFFSET)	   /* SIGNED BIGINT */
#define SQL_C_UBIGINT	(SQL_BIGINT+SQL_UNSIGNED_OFFSET)   /* UNSIGNED BIGINT */


#define SQL_C_VARBOOKMARK			SQL_C_BINARY

/* define for SQL_DIAG_ROW_NUMBER and SQL_DIAG_COLUMN_NUMBER */
#define SQL_NO_ROW_NUMBER			(-1)
#define SQL_NO_COLUMN_NUMBER			(-1)
#define SQL_ROW_NUMBER_UNKNOWN			(-2)
#define SQL_COLUMN_NUMBER_UNKNOWN		(-2)

/* SQLBindParameter extensions */
#define SQL_DEFAULT_PARAM           		(-5)
#define SQL_IGNORE                   		(-6)
#define SQL_COLUMN_IGNORE			SQL_IGNORE

/* binary length for driver specific attributes */
#define SQL_LEN_BINARY_ATTR_OFFSET	 	(-100)
#define SQL_LEN_BINARY_ATTR(length)	 (-(length)+SQL_LEN_BINARY_ATTR_OFFSET)

/* Defines used by Driver Manager when mapping SQLSetParam to SQLBindParameter
*/
#define SQL_PARAM_TYPE_DEFAULT           	SQL_PARAM_INPUT_OUTPUT
#define SQL_SETPARAM_VALUE_MAX           	(-1L)

/* SQLColAttributes defines */
#define SQL_COLUMN_COUNT                	0
#define SQL_COLUMN_NAME                 	1
#define SQL_COLUMN_TYPE                 	2
#define SQL_COLUMN_LENGTH               	3
#define SQL_COLUMN_PRECISION            	4
#define SQL_COLUMN_SCALE                	5
#define SQL_COLUMN_DISPLAY_SIZE         	6
#define SQL_COLUMN_NULLABLE             	7

/* SQLColAttributes subdefines for SQL_COLUMN_SEARCHABLE */
/* These are also used by SQLGetInfo                     */
#define SQL_PRED_SEARCHABLE			SQL_SEARCHABLE

/********************************************/
/* SQLGetFunctions: additional values for   */
/* fFunction to represent functions that    */
/* are not in the X/Open spec.		    */
/********************************************/

#define SQL_API_SQLALLOCHANDLESTD		73
#define SQL_API_SQLBULKOPERATIONS		24

/*----------------------------------------------*/
/* 2.X drivers export a dummy function with  	*/
/* ordinal number SQL_API_LOADBYORDINAL to speed*/
/* loading under the windows operating system.  */
/* 						*/
/* CAUTION: Loading by ordinal is not supported */
/* for 3.0 and above drivers.			*/
/*----------------------------------------------*/

#define SQL_API_LOADBYORDINAL       		199	/* See CAUTION above */	

/*----------------------------------------------*/
/* SQL_API_ODBC3_ALL_FUNCTIONS                  */
/* This returns a bitmap, which allows us to    */
/* handle the higher-valued function numbers.   */
/* Use  SQL_FUNC_EXISTS(bitmap,function_number) */
/* to determine if the function exists.         */
/*----------------------------------------------*/


#define SQL_API_ODBC3_ALL_FUNCTIONS		999
#define	SQL_API_ODBC3_ALL_FUNCTIONS_SIZE	250	/* array of 250 words */

#define SQL_FUNC_EXISTS(pfExists, uwAPI) \
				((*(((UWORD*) (pfExists)) + ((uwAPI) >> 4)) \
					& (1 << ((uwAPI) & 0x000F)) \
 				 ) ? SQL_TRUE : SQL_FALSE \
				)


/* SQL_ALTER_TABLE bitmasks */
/* the following 5 bitmasks are defined in sql.h
*#define SQL_AT_ADD_COLUMN                   	0x00000001L
*#define SQL_AT_DROP_COLUMN                  	0x00000002L
*/
#define SQL_AT_ADD_CONSTRAINT               	0x00000008L

#define	SQL_AT_ADD_COLUMN_SINGLE		0x00000020L	
#define	SQL_AT_ADD_COLUMN_DEFAULT		0x00000040L
#define	SQL_AT_ADD_COLUMN_COLLATION		0x00000080L
#define	SQL_AT_SET_COLUMN_DEFAULT		0x00000100L
#define	SQL_AT_DROP_COLUMN_DEFAULT		0x00000200L
#define	SQL_AT_DROP_COLUMN_CASCADE		0x00000400L
#define	SQL_AT_DROP_COLUMN_RESTRICT		0x00000800L
#define SQL_AT_ADD_TABLE_CONSTRAINT		0x00001000L		
#define SQL_AT_DROP_TABLE_CONSTRAINT_CASCADE	0x00002000L		
#define SQL_AT_DROP_TABLE_CONSTRAINT_RESTRICT	0x00004000L		
#define SQL_AT_CONSTRAINT_NAME_DEFINITION	0x00008000L
#define SQL_AT_CONSTRAINT_INITIALLY_DEFERRED	0x00010000L
#define SQL_AT_CONSTRAINT_INITIALLY_IMMEDIATE	0x00020000L
#define SQL_AT_CONSTRAINT_DEFERRABLE		0x00040000L
#define SQL_AT_CONSTRAINT_NON_DEFERRABLE	0x00080000L

/* SQL_CONVERT_*  return value bitmasks */
#define SQL_CVT_INTERVAL_YEAR_MONTH	    	0x00080000L
#define SQL_CVT_INTERVAL_DAY_TIME	    	0x00100000L
#define	SQL_CVT_WCHAR				0x00200000L
#define	SQL_CVT_WLONGVARCHAR			0x00400000L
#define	SQL_CVT_WVARCHAR			0x00800000L


/* SQL_CONVERT_FUNCTIONS functions */
#define SQL_FN_CVT_CAST				0x00000002L


/* SQL_STRING_FUNCTIONS functions */
#define SQL_FN_STR_BIT_LENGTH			0x00080000L
#define SQL_FN_STR_CHAR_LENGTH			0x00100000L
#define SQL_FN_STR_CHARACTER_LENGTH		0x00200000L
#define SQL_FN_STR_OCTET_LENGTH			0x00400000L
#define SQL_FN_STR_POSITION			0x00800000L

/* SQL_SQL92_STRING_FUNCTIONS */
#define SQL_SSF_CONVERT				0x00000001L	
#define SQL_SSF_LOWER				0x00000002L
#define SQL_SSF_UPPER				0x00000004L
#define SQL_SSF_SUBSTRING			0x00000008L
#define SQL_SSF_TRANSLATE			0x00000010L
#define SQL_SSF_TRIM_BOTH			0x00000020L
#define SQL_SSF_TRIM_LEADING			0x00000040L
#define SQL_SSF_TRIM_TRAILING			0x00000080L

/* SQL_NUMERIC_FUNCTIONS functions */

/* SQL_SQL92_NUMERIC_VALUE_FUNCTIONS */
#define SQL_SNVF_BIT_LENGTH			0x00000001L
#define SQL_SNVF_CHAR_LENGTH			0x00000002L
#define SQL_SNVF_CHARACTER_LENGTH		0x00000004L
#define SQL_SNVF_EXTRACT			0x00000008L
#define SQL_SNVF_OCTET_LENGTH			0x00000010L
#define SQL_SNVF_POSITION			0x00000020L

/* SQL_TIMEDATE_FUNCTIONS functions */
#define SQL_FN_TD_CURRENT_DATE			0x00020000L
#define SQL_FN_TD_CURRENT_TIME			0x00040000L
#define SQL_FN_TD_CURRENT_TIMESTAMP		0x00080000L
#define SQL_FN_TD_EXTRACT			0x00100000L

/* SQL_SQL92_DATETIME_FUNCTIONS */
#define SQL_SDF_CURRENT_DATE			0x00000001L
#define SQL_SDF_CURRENT_TIME			0x00000002L
#define SQL_SDF_CURRENT_TIMESTAMP		0x00000004L


/* bitmasks for SQL_DYNAMIC_CURSOR_ATTRIBUTES1,
 * SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1, 
 * SQL_KEYSET_CURSOR_ATTRIBUTES1, and SQL_STATIC_CURSOR_ATTRIBUTES1 
 */
/* supported SQLFetchScroll FetchOrientation's */
#define SQL_CA1_NEXT				0x00000001L
#define SQL_CA1_ABSOLUTE			0x00000002L
#define SQL_CA1_RELATIVE			0x00000004L
#define SQL_CA1_BOOKMARK			0x00000008L

/* supported SQLSetPos LockType's */
#define SQL_CA1_LOCK_NO_CHANGE			0x00000040L
#define SQL_CA1_LOCK_EXCLUSIVE			0x00000080L
#define SQL_CA1_LOCK_UNLOCK			0x00000100L

/* supported SQLSetPos Operations */
#define SQL_CA1_POS_POSITION			0x00000200L
#define SQL_CA1_POS_UPDATE			0x00000400L
#define SQL_CA1_POS_DELETE			0x00000800L
#define SQL_CA1_POS_REFRESH			0x00001000L

/* positioned updates and deletes */
#define SQL_CA1_POSITIONED_UPDATE		0x00002000L
#define SQL_CA1_POSITIONED_DELETE		0x00004000L
#define SQL_CA1_SELECT_FOR_UPDATE		0x00008000L

/* supported SQLBulkOperations operations */
#define SQL_CA1_BULK_ADD			0x00010000L
#define SQL_CA1_BULK_UPDATE_BY_BOOKMARK		0x00020000L
#define SQL_CA1_BULK_DELETE_BY_BOOKMARK		0x00040000L
#define SQL_CA1_BULK_FETCH_BY_BOOKMARK		0x00080000L

/* bitmasks for SQL_DYNAMIC_CURSOR_ATTRIBUTES2,
 * SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2, 
 * SQL_KEYSET_CURSOR_ATTRIBUTES2, and SQL_STATIC_CURSOR_ATTRIBUTES2 
 */
/* supported values for SQL_ATTR_SCROLL_CONCURRENCY */
#define SQL_CA2_READ_ONLY_CONCURRENCY		0x00000001L
#define SQL_CA2_LOCK_CONCURRENCY		0x00000002L
#define SQL_CA2_OPT_ROWVER_CONCURRENCY		0x00000004L
#define SQL_CA2_OPT_VALUES_CONCURRENCY		0x00000008L

/* sensitivity of the cursor to its own inserts, deletes, and updates */
#define SQL_CA2_SENSITIVITY_ADDITIONS		0x00000010L
#define SQL_CA2_SENSITIVITY_DELETIONS		0x00000020L
#define SQL_CA2_SENSITIVITY_UPDATES		0x00000040L

/* semantics of SQL_ATTR_MAX_ROWS */
#define SQL_CA2_MAX_ROWS_SELECT			0x00000080L
#define SQL_CA2_MAX_ROWS_INSERT			0x00000100L
#define SQL_CA2_MAX_ROWS_DELETE			0x00000200L
#define SQL_CA2_MAX_ROWS_UPDATE			0x00000400L
#define SQL_CA2_MAX_ROWS_CATALOG		0x00000800L
#define SQL_CA2_MAX_ROWS_AFFECTS_ALL		(SQL_CA2_MAX_ROWS_SELECT | \
						 SQL_CA2_MAX_ROWS_INSERT | \
						 SQL_CA2_MAX_ROWS_DELETE | \
						 SQL_CA2_MAX_ROWS_UPDATE | \
						 SQL_CA2_MAX_ROWS_CATALOG)

/* semantics of SQL_DIAG_CURSOR_ROW_COUNT */
#define SQL_CA2_CRC_EXACT			0x00001000L
#define SQL_CA2_CRC_APPROXIMATE			0x00002000L

/* the kinds of positioned statements that can be simulated */
#define SQL_CA2_SIMULATE_NON_UNIQUE		0x00004000L
#define SQL_CA2_SIMULATE_TRY_UNIQUE		0x00008000L
#define SQL_CA2_SIMULATE_UNIQUE			0x00010000L

/* SQL_FILE_USAGE values */

#define SQL_FILE_CATALOG			SQL_FILE_QUALIFIER // ODBC 3.0


/* SQL_GROUP_BY values */
#define	SQL_GB_COLLATE				0x0004

/* SQL_SCHEMA_USAGE masks */
#define SQL_SU_DML_STATEMENTS			SQL_OU_DML_STATEMENTS 
#define SQL_SU_PROCEDURE_INVOCATION		SQL_OU_PROCEDURE_INVOCATION
#define SQL_SU_TABLE_DEFINITION			SQL_OU_TABLE_DEFINITION
#define SQL_SU_INDEX_DEFINITION			SQL_OU_INDEX_DEFINITION
#define SQL_SU_PRIVILEGE_DEFINITION		SQL_OU_PRIVILEGE_DEFINITION

/* SQL_CATALOG_USAGE masks */
#define SQL_CU_DML_STATEMENTS			SQL_QU_DML_STATEMENTS
#define SQL_CU_PROCEDURE_INVOCATION		SQL_QU_PROCEDURE_INVOCATION 
#define SQL_CU_TABLE_DEFINITION			SQL_QU_TABLE_DEFINITION
#define SQL_CU_INDEX_DEFINITION			SQL_QU_INDEX_DEFINITION 
#define SQL_CU_PRIVILEGE_DEFINITION		SQL_QU_PRIVILEGE_DEFINITION 

/* SQL_VIEW values */
#define	SQL_CV_CREATE_VIEW			0x00000001L
#define	SQL_CV_CHECK_OPTION			0x00000002L
#define	SQL_CV_CASCADED				0x00000004L
#define	SQL_CV_LOCAL				0x00000008L

/* Here start return values for ODBC 3.0 SQLGetInfo */

/* additional SQLDataSources fetch directions */
#define SQL_FETCH_FIRST_USER			31
#define SQL_FETCH_FIRST_SYSTEM			32

/* Defines for SQLTables */
#define SQL_ALL_CATALOGS			"%"
#define SQL_ALL_SCHEMAS				"%"
#define SQL_ALL_TABLE_TYPES			"%"

/* Level 2 Functions                             */
#define SQL_ROW_SUCCESS_WITH_INFO		6
#define SQL_ROW_PROCEED				0
#define SQL_ROW_IGNORE				1

/* value for SQL_DESC_ARRAY_STATUS_PTR */
#define SQL_PARAM_SUCCESS			0
#define SQL_PARAM_SUCCESS_WITH_INFO		6
#define SQL_PARAM_ERROR				5
#define SQL_PARAM_UNUSED			7
#define SQL_PARAM_DIAG_UNAVAILABLE		1

#define SQL_PARAM_PROCEED			0
#define SQL_PARAM_IGNORE			1

/* Defines for SQLForeignKeys (returned in result set) */
#define SQL_INITIALLY_DEFERRED			5
#define SQL_INITIALLY_IMMEDIATE			6

/*
 * enumerations for DATETIME_INTERVAL_SUBCODE values for interval data types
 * these values are from SQL-92
 */
typedef enum
  {
    SQL_IS_YEAR = 1,
    SQL_IS_MONTH = 2,
    SQL_IS_DAY = 3,
    SQL_IS_HOUR = 4,
    SQL_IS_MINUTE = 5,
    SQL_IS_SECOND = 6,
    SQL_IS_YEAR_TO_MONTH = 7,
    SQL_IS_DAY_TO_HOUR = 8,
    SQL_IS_DAY_TO_MINUTE = 9,
    SQL_IS_DAY_TO_SECOND = 10,
    SQL_IS_HOUR_TO_MINUTE = 11,
    SQL_IS_HOUR_TO_SECOND = 12,
    SQL_IS_MINUTE_TO_SECOND = 13
  }
SQLINTERVAL;

typedef struct tagSQL_YEAR_MONTH
  {
    SQLUINTEGER year;
    SQLUINTEGER month;
  }
SQL_YEAR_MONTH_STRUCT;

typedef struct tagSQL_DAY_SECOND
  {
    SQLUINTEGER day;
    SQLUINTEGER hour;
    SQLUINTEGER minute;
    SQLUINTEGER second;
    SQLUINTEGER fraction;
  }
SQL_DAY_SECOND_STRUCT;

typedef struct tagSQL_INTERVAL_STRUCT
  {
    SQLINTERVAL interval_type;
    SQLSMALLINT interval_sign;
    union
      {
	SQL_YEAR_MONTH_STRUCT year_month;
	SQL_DAY_SECOND_STRUCT day_second;
      }
    intval;
  }
SQL_INTERVAL_STRUCT;

#define SQL_DATE_STRUCT		DATE_STRUCT
#define	SQL_TIME_STRUCT		TIME_STRUCT
#define SQL_TIMESTAMP_STRUCT	TIMESTAMP_STRUCT

/* the ODBC C types for SQL_C_SBIGINT and SQL_C_UBIGINT */
#if (_MSC_VER >= 900)
#define ODBCINT64	__int64
#endif

/* 
 *  If using other compilers, define ODBCINT64 to the 
 *  approriate 64 bit integer type 
 */
#ifdef ODBCINT64
typedef ODBCINT64 SQLBIGINT;
typedef unsigned ODBCINT64 SQLUBIGINT;
#endif

/* internal representation of numeric data type */
#define SQL_MAX_NUMERIC_LEN		16
typedef struct tagSQL_NUMERIC_STRUCT
  {
    SQLCHAR precision;
    SQLSCHAR scale;
    SQLCHAR sign;		/* 1 if positive, 0 if negative */
    SQLCHAR val[SQL_MAX_NUMERIC_LEN];
  }
SQL_NUMERIC_STRUCT;

#define SQL_DTC_TRANSITION_COST			1750

/* SQL_DTC_TRANSITION_COST bitmasks */
#define SQL_DTC_ENLIST_EXPENSIVE                0x00000001L
#define SQL_DTC_UNENLIST_EXPENSIVE              0x00000002L

/* Operations in SQLBulkOperations */
#define SQL_UPDATE_BY_BOOKMARK		 5
#define SQL_DELETE_BY_BOOKMARK		 6
#define	SQL_FETCH_BY_BOOKMARK		 7

#endif /* UDBCODB3*/
