/*
 *  udbcodbc.h
 *
 *  $Id: udbcodbc.h,v 1.22 2001/04/10 14:40:21 source Exp $
 *
 *  Microsoft ODBC extensions and changes to the SAG specifications
 *
 *  (C)Copyright 1993, 1994 OpenLink Software.
 *  All Rights Reserved.
 *
 *  The copyright above and this notice must be preserved in all
 *  copies of this source code.  The copyright above does not
 *  evidence any actual or intended publication of this source code.
 *
 *  This is unpublished proprietary trade secret of OpenLink Software.
 *  This source code may not be copied, disclosed, distributed, demonstrated
 *  or licensed except as authorized by OpenLink Software.
 */

#ifndef _UDBCODBC_H
#define _UDBCODBC_H

/*
 * Set default specification to ODBC 3.50
 */
#ifndef ODBCVER
#define ODBCVER 0x0350
#endif

#ifndef _UDBCSAG_H
#include <udbc/udbcsag.h>
#endif

/*
 *  ODBC data types
 */
#define HENV 					SQLHENV
#define HDBC 					SQLHDBC
#define HSTMT 					SQLHSTMT
#if (ODBCVER >= 0x0300)
#define HDESC					SQLHDESC
#endif
#define RETCODE 				SQLRETURN

#define SQL_API 				UDBC_API

/*
 *  Extra indicator values
 */
#define SQL_DATA_AT_EXEC 			(-2)

/*
 * Generally useful constants
 */
#define SQL_MAX_OPTION_STRING_LENGTH 		256

/*
 *  Extra return codes
 */
#define SQL_STILL_EXECUTING 			2
#define SQL_NEED_DATA 				99
#define SQL_NO_DATA_FOUND 			SQL_NO_DATA

/*
 *  Extended datatypes
 */
#define SQL_DATE 				9
#define SQL_TIME 				10
#define SQL_TIMESTAMP 				11
#define SQL_LONGVARCHAR 			(-1)
#define SQL_BINARY 				(-2)
#define SQL_VARBINARY 				(-3)
#define SQL_LONGVARBINARY 			(-4)
#define SQL_BIGINT 				(-5)
#define SQL_TINYINT 				(-6)
#define SQL_BIT 				(-7)
#if (ODBCVER >= 0x0350)
#define SQL_GUID				(-11)
#endif  /* ODBCVER >= 0x0350 */

#if (ODBCVER < 0x0300)
/*
 *  These are ODBC 3.0 INTERNAL defines, so should not be used in an
 *  ODBC 2.x environment. They are here for building a ODBC 2.x UDBC 
 *  library.
 */
/* SQL data type codes */
#define	SQL_UNKNOWN_TYPE			0
#define SQL_DATETIME        			9

/* One-parameter shortcuts for date/time data types */
#define SQL_TYPE_DATE      			91
#define SQL_TYPE_TIME      			92
#define SQL_TYPE_TIMESTAMP 			93

/* SQL date/time type subcodes */
#define SQL_CODE_DATE       			1
#define SQL_CODE_TIME       			2
#define SQL_CODE_TIMESTAMP  			3
#endif

#ifdef NOT_IMPLEMENTED
#define SQL_INTERVAL_YEAR 			(-80)
#define SQL_INTERVAL_MONTH 			(-81)
#define SQL_INTERVAL_YEAR_TO_MONTH 		(-82)
#define SQL_INTERVAL_DAY 			(-83)
#define SQL_INTERVAL_HOUR 			(-84)
#define SQL_INTERVAL_MINUTE 			(-85)
#define SQL_INTERVAL_SECOND 			(-86)
#define SQL_INTERVAL_DAY_TO_HOUR 		(-87)
#define SQL_INTERVAL_DAY_TO_MINUTE 		(-88)
#define SQL_INTERVAL_DAY_TO_SECOND 		(-89)
#define SQL_INTERVAL_HOUR_TO_MINUTE 		(-90)
#define SQL_INTERVAL_HOUR_TO_SECOND 		(-91)
#define SQL_INTERVAL_MINUTE_TO_SECOND 		(-92)
#define SQL_UNICODE 				(-95)
#define SQL_UNICODE_VARCHAR 			(-96)
#define SQL_UNICODE_LONGVARCHAR 		(-97)
#define SQL_UNICODE_CHAR 			SQL_UNICODE

#define SQL_TYPE_DRIVER_START 			SQL_INTERVAL_YEAR
#define SQL_TYPE_DRIVER_END 			SQL_UNICODE_LONGVARCHAR
#endif /* NOT IMPLEMENTED */

/*
 *  Other names for host variable types
 */
#define SQL_C_DEFAULT 				SQLBUF_DEFAULT
#define SQL_C_CHAR 				SQLBUF_CHAR
#define SQL_C_LONG 				SQLBUF_LONG
#define SQL_C_SHORT 				SQLBUF_SHORT
#define SQL_C_FLOAT 				SQLBUF_FLOAT
#define SQL_C_DOUBLE 				SQLBUF_DOUBLE

#define SQL_C_DATE 				SQL_DATE
#define SQL_C_TIME 				SQL_TIME
#define SQL_C_TIMESTAMP 			SQL_TIMESTAMP
#define SQL_C_BINARY 				SQL_BINARY
#define SQL_C_BIT 				SQL_BIT
#define SQL_C_TINYINT 				SQL_TINYINT

#define SQL_SIGNED_OFFSET 			(-20)
#define SQL_UNSIGNED_OFFSET 			(-22)

#define SQL_C_SLONG 	SQL_C_LONG+SQL_SIGNED_OFFSET 	/* SIGNED INTEGER */
#define SQL_C_SSHORT 	SQL_C_SHORT+SQL_SIGNED_OFFSET 	/* SIGNED SMALLINT */
#define SQL_C_STINYINT 	SQL_TINYINT+SQL_SIGNED_OFFSET 	/* SIGNED TINYINT */
#define SQL_C_ULONG 	SQL_C_LONG+SQL_UNSIGNED_OFFSET 	/* UNSIGNED INTEGER */
#define SQL_C_USHORT 	SQL_C_SHORT+SQL_UNSIGNED_OFFSET /* UNSIGNED SMALLINT*/
#define SQL_C_UTINYINT 	SQL_TINYINT+SQL_UNSIGNED_OFFSET /* UNSIGNED TINYINT */
#define SQL_C_BOOKMARK 	SQL_C_ULONG 			/* BOOKMARK */

#if (ODBCVER >= 0x0350)
#define SQL_C_GUID				SQL_GUID
#endif  /* ODBCVER >= 0x0350 */

#define SQL_ALL_TYPES 				0
#define SQL_TYPE_NULL 				0
#define SQL_TYPE_MIN 				SQL_BIT
#define SQL_TYPE_MAX 				SQL_VARCHAR

/*
 *  Transfer types for DATE, TIME, TIMESTAMP
 */
typedef struct tagDATE_STRUCT
  {
    SWORD year;
    UWORD month;
    UWORD day;
  }
DATE_STRUCT;

typedef struct tagTIME_SRUCT
  {
    UWORD hour;
    UWORD minute;
    UWORD second;
  }
TIME_STRUCT;
 
typedef UDWORD BOOKMARK;

typedef struct tagTIMESTAMP_STRUCT
  {
    SWORD year;
    UWORD month;
    UWORD day;
    UWORD hour;
    UWORD minute;
    UWORD second;
    UDWORD fraction;
  }
TIMESTAMP_STRUCT;

/*
 *  Special return values for SQLGetData
 */
#define SQL_NO_TOTAL 				(-4)

/*
 *  Extra options for SQLColAttributes
 */
#define SQL_COLUMN_UNSIGNED 			8
#define SQL_COLUMN_MONEY 			9
#define SQL_COLUMN_UPDATABLE 			10
#define SQL_COLUMN_AUTO_INCREMENT 		11
#define SQL_COLUMN_CASE_SENSITIVE 		12
#define SQL_COLUMN_SEARCHABLE 			13
#define SQL_COLUMN_TYPE_NAME 			14
#define SQL_COLUMN_TABLE_NAME           	15
#define SQL_COLUMN_OWNER_NAME           	16
#define SQL_COLUMN_QUALIFIER_NAME       	17
#define SQL_COLUMN_LABEL                	18

#define SQL_COLATT_OPT_MIN              	SQL_COLUMN_COUNT               
#define SQL_COLATT_OPT_MAX              	SQL_COLUMN_LABEL
#define SQL_COLUMN_DRIVER_START         	1000

/*
 *  SQLColAttributes subdefines for SQL_COLUMN_UPDATABLE
 */
#define SQL_ATTR_READONLY 			0
#define SQL_ATTR_WRITE 				1
#define SQL_ATTR_READWRITE_UNKNOWN 		2

/*
 *  SQLColAttributes subdefines for SQL_COLUMN_SEARCHABLE
 *  (Also used by SQLGetInfo)
 */
#define SQL_UNSEARCHABLE 			0
#define SQL_LIKE_ONLY 				1
#define SQL_ALL_EXCEPT_LIKE 			2
#define SQL_SEARCHABLE 				3

/*
 *  Access modes
 */
#define SQL_MODE_READ_WRITE 			0
#define SQL_MODE_READ_ONLY 			1
#define SQL_MODE_DEFAULT 			SQL_MODE_READ_WRITE

/*
 * Autocommit mode
 */
#define SQL_AUTOCOMMIT_OFF 			0
#define SQL_AUTOCOMMIT_ON 			1
#define SQL_AUTOCOMMIT_DEFAULT 			SQL_AUTOCOMMIT_ON

/*
 *  Options for SQLDriverConnect
 */
#define SQL_DRIVER_NOPROMPT 			0
#define SQL_DRIVER_COMPLETE 			1
#define SQL_DRIVER_PROMPT 			2
#define SQL_DRIVER_COMPLETE_REQUIRED 		3

/*
 *  Defines for SQLGetFunctions
 */
#define SQL_API_ALL_FUNCTIONS 			0

#define SQL_API_SQLALLOCCONNECT 		1
#define SQL_API_SQLALLOCENV 			2
#define SQL_API_SQLALLOCSTMT 			3
#define SQL_API_SQLBINDCOL 			4
#define SQL_API_SQLCANCEL 			5
#define SQL_API_SQLCOLATTRIBUTES 		6
#define SQL_API_SQLCONNECT 			7
#define SQL_API_SQLDESCRIBECOL 			8
#define SQL_API_SQLDISCONNECT 			9
#define SQL_API_SQLERROR 			10
#define SQL_API_SQLEXECDIRECT 			11
#define SQL_API_SQLEXECUTE 			12
#define SQL_API_SQLFETCH 			13
#define SQL_API_SQLFREECONNECT 			14
#define SQL_API_SQLFREEENV 			15
#define SQL_API_SQLFREESTMT 			16
#define SQL_API_SQLGETCURSORNAME 		17
#define SQL_API_SQLNUMRESULTCOLS 		18
#define SQL_API_SQLPREPARE 			19
#define SQL_API_SQLROWCOUNT 			20
#define SQL_API_SQLSETCURSORNAME 		21
#define SQL_API_SQLSETPARAM 			22
#define SQL_API_SQLTRANSACT 			23

#define SQL_NUM_FUNCTIONS 			23

#define SQL_EXT_API_START 			40

#define SQL_API_SQLCOLUMNS 			40
#define SQL_API_SQLDRIVERCONNECT 		41
#define SQL_API_SQLGETCONNECTOPTION 		42
#define SQL_API_SQLGETDATA 			43
#define SQL_API_SQLGETFUNCTIONS 		44
#define SQL_API_SQLGETINFO 			45
#define SQL_API_SQLGETSTMTOPTION 		46
#define SQL_API_SQLGETTYPEINFO 			47
#define SQL_API_SQLPARAMDATA 			48
#define SQL_API_SQLPUTDATA 			49
#define SQL_API_SQLSETCONNECTOPTION 		50
#define SQL_API_SQLSETSTMTOPTION 		51
#define SQL_API_SQLSPECIALCOLUMNS 		52
#define SQL_API_SQLSTATISTICS 			53
#define SQL_API_SQLTABLES 			54

#define SQL_API_SQLBROWSECONNECT 		55
#define SQL_API_SQLCOLUMNPRIVILEGES 		56
#define SQL_API_SQLDATASOURCES 			57
#define SQL_API_SQLDESCRIBEPARAM 		58
#define SQL_API_SQLEXTENDEDFETCH 		59
#define SQL_API_SQLFOREIGNKEYS 			60
#define SQL_API_SQLMORERESULTS 			61
#define SQL_API_SQLNATIVESQL 			62
#define SQL_API_SQLNUMPARAMS 			63
#define SQL_API_SQLPARAMOPTIONS 		64
#define SQL_API_SQLPRIMARYKEYS 			65
#define SQL_API_SQLPROCEDURECOLUMNS 		66
#define SQL_API_SQLPROCEDURES 			67
#define SQL_API_SQLSETPOS 			68
#define SQL_API_SQLSETSCROLLOPTIONS 		69
#define SQL_API_SQLTABLEPRIVILEGES 		70

/*
 *  SDK 2.0 Additions 		
 */
#define SQL_API_SQLDRIVERS 			71
#define SQL_API_SQLBINDPARAMETER 		72

#define SQL_EXT_API_LAST 			SQL_API_SQLBINDPARAMETER
#define SQL_NUM_EXTENSIONS  (SQL_EXT_API_LAST-SQL_EXT_API_START+1)

/*
 *  Defines for SQLGetInfo
 */
#define SQL_INFO_FIRST 				0
#define SQL_ACTIVE_CONNECTIONS 			0
#define SQL_ACTIVE_STATEMENTS 			1
#define SQL_DATA_SOURCE_NAME 			2
#define SQL_DRIVER_HDBC 			3
#define SQL_DRIVER_HENV 			4
#define SQL_DRIVER_HSTMT 			5
#define SQL_DRIVER_NAME 			6
#define SQL_DRIVER_VER 				7
#define SQL_FETCH_DIRECTION 			8
#define SQL_ODBC_API_CONFORMANCE 		9
#define SQL_ODBC_VER 				10
#define SQL_ROW_UPDATES 			11
#define SQL_ODBC_SAG_CLI_CONFORMANCE 		12
#define SQL_SERVER_NAME 			13
#define SQL_SEARCH_PATTERN_ESCAPE 		14
#define SQL_ODBC_SQL_CONFORMANCE 		15
#define SQL_DATABASE_NAME 			16
#define SQL_DBMS_NAME 				17
#define SQL_DBMS_VER 				18
#define SQL_ACCESSIBLE_TABLES 			19
#define SQL_ACCESSIBLE_PROCEDURES 		20
#define SQL_PROCEDURES 				21
#define SQL_CONCAT_NULL_BEHAVIOR 		22
#define SQL_CURSOR_COMMIT_BEHAVIOR 		23
#define SQL_CURSOR_ROLLBACK_BEHAVIOR 		24
#define SQL_DATA_SOURCE_READ_ONLY 		25
#define SQL_DEFAULT_TXN_ISOLATION 		26
#define SQL_EXPRESSIONS_IN_ORDERBY 		27
#define SQL_IDENTIFIER_CASE 			28
#define SQL_IDENTIFIER_QUOTE_CHAR 		29
#define SQL_MAX_COLUMN_NAME_LEN 		30
#define SQL_MAX_CURSOR_NAME_LEN 		31
#define SQL_MAX_OWNER_NAME_LEN 			32
#define SQL_MAX_PROCEDURE_NAME_LEN 		33
#define SQL_MAX_QUALIFIER_NAME_LEN 		34
#define SQL_MAX_TABLE_NAME_LEN 			35
#define SQL_MULT_RESULT_SETS 			36
#define SQL_MULTIPLE_ACTIVE_TXN 		37
#define SQL_OUTER_JOINS 			38
#define SQL_OWNER_TERM 				39
#define SQL_PROCEDURE_TERM 			40
#define SQL_QUALIFIER_NAME_SEPARATOR 		41
#define SQL_QUALIFIER_TERM 			42
#define SQL_SCROLL_CONCURRENCY 			43
#define SQL_SCROLL_OPTIONS 			44
#define SQL_TABLE_TERM 				45
#define SQL_TXN_CAPABLE 			46
#define SQL_USER_NAME 				47
#define SQL_CONVERT_FUNCTIONS 			48
#define SQL_NUMERIC_FUNCTIONS 			49
#define SQL_STRING_FUNCTIONS 			50
#define SQL_SYSTEM_FUNCTIONS 			51
#define SQL_TIMEDATE_FUNCTIONS 			52
#define SQL_CONVERT_BIGINT 			53
#define SQL_CONVERT_BINARY 			54
#define SQL_CONVERT_BIT 			55
#define SQL_CONVERT_CHAR 			56
#define SQL_CONVERT_DATE 			57
#define SQL_CONVERT_DECIMAL 			58
#define SQL_CONVERT_DOUBLE 			59
#define SQL_CONVERT_FLOAT 			60
#define SQL_CONVERT_INTEGER 			61
#define SQL_CONVERT_LONGVARCHAR 		62
#define SQL_CONVERT_NUMERIC 			63
#define SQL_CONVERT_REAL 			64
#define SQL_CONVERT_SMALLINT 			65
#define SQL_CONVERT_TIME 			66
#define SQL_CONVERT_TIMESTAMP 			67
#define SQL_CONVERT_TINYINT 			68
#define SQL_CONVERT_VARBINARY 			69
#define SQL_CONVERT_VARCHAR 			70
#define SQL_CONVERT_LONGVARBINARY 		71
#define SQL_TXN_ISOLATION_OPTION 		72
#define SQL_ODBC_SQL_OPT_IEF 			73

#define SQL_CORRELATION_NAME 			74
#define SQL_NON_NULLABLE_COLUMNS 		75

/*** ODBC SDK 2.0 Additions ***/
#define SQL_DRIVER_HLIB 			76
#define SQL_DRIVER_ODBC_VER 			77
#define SQL_LOCK_TYPES 				78
#define SQL_POS_OPERATIONS 			79
#define SQL_POSITIONED_STATEMENTS 		80
#define SQL_GETDATA_EXTENSIONS 			81
#define SQL_BOOKMARK_PERSISTENCE 		82
#define SQL_STATIC_SENSITIVITY 			83
#define SQL_FILE_USAGE 				84
#define SQL_NULL_COLLATION 			85
#define SQL_ALTER_TABLE 			86
#define SQL_COLUMN_ALIAS 			87
#define SQL_GROUP_BY 				88
#define SQL_KEYWORDS 				89
#define SQL_ORDER_BY_COLUMNS_IN_SELECT 		90
#define SQL_OWNER_USAGE 			91
#define SQL_QUALIFIER_USAGE 			92
#define SQL_QUOTED_IDENTIFIER_CASE 		93
#define SQL_SPECIAL_CHARACTERS 			94
#define SQL_SUBQUERIES 				95
#define SQL_UNION 				96
#define SQL_MAX_COLUMNS_IN_GROUP_BY 		97
#define SQL_MAX_COLUMNS_IN_INDEX 		98
#define SQL_MAX_COLUMNS_IN_ORDER_BY 		99
#define SQL_MAX_COLUMNS_IN_SELECT 		100
#define SQL_MAX_COLUMNS_IN_TABLE 		101
#define SQL_MAX_INDEX_SIZE 			102
#define SQL_MAX_ROW_SIZE_INCLUDES_LONG 		103
#define SQL_MAX_ROW_SIZE 			104
#define SQL_MAX_STATEMENT_LEN 			105
#define SQL_MAX_TABLES_IN_SELECT 		106
#define SQL_MAX_USER_NAME_LEN 			107
#define SQL_MAX_CHAR_LITERAL_LEN 		108
#define SQL_TIMEDATE_ADD_INTERVALS 		109
#define SQL_TIMEDATE_DIFF_INTERVALS 		110
#define SQL_NEED_LONG_DATA_LEN 			111
#define SQL_MAX_BINARY_LITERAL_LEN 		112
#define SQL_LIKE_ESCAPE_CLAUSE 			113
#define SQL_QUALIFIER_LOCATION 			114

#if (ODBCVER < 0x0300)
#define SQL_INFO_LAST 				SQL_QUALIFIER_LOCATION
#define SQL_INFO_DRIVER_START 			1000
#endif

#if (ODBCVER < 0x0300)
#define SQL_OJ_CAPABILITIES 			65003 
#else
#define SQL_OJ_CAPABILITIES 			115
#define SQL_OUTER_JOIN_CAPABILITIES 		SQL_OJ_CAPABILITIES
#endif

/*
 *  ODBC 3.0 SQLGetInfo values
 */
#if (ODBCVER >= 0x0300)
#define SQL_ACTIVE_ENVIRONMENTS 		116
#define SQL_ALTER_DOMAIN 			117
#define SQL_SQL_CONFORMANCE 			118
#define SQL_DATETIME_LITERALS 			119
#define SQL_BATCH_ROW_COUNT 			120
#define SQL_BATCH_SUPPORT 			121
#define SQL_CATALOG_LOCATION 			SQL_QUALIFIER_LOCATION
#define SQL_CATALOG_NAME_SEPARATOR 		SQL_QUALIFIER_NAME_SEPARATOR
#define SQL_CATALOG_TERM 			SQL_QUALIFIER_TERM
#define SQL_CATALOG_USAGE 			SQL_QUALIFIER_USAGE
#define SQL_CONVERT_WCHAR 			122
#define SQL_CONVERT_INTERVAL_DAY_TIME 		123
#define SQL_CONVERT_INTERVAL_YEAR_MONTH 	124
#define SQL_CONVERT_WLONGVARCHAR 		125
#define SQL_CONVERT_WVARCHAR 			126
#define SQL_CREATE_ASSERTION 			127
#define SQL_CREATE_CHARACTER_SET 		128
#define SQL_CREATE_COLLATION 			129
#define SQL_CREATE_DOMAIN 			130
#define SQL_CREATE_SCHEMA 			131
#define SQL_CREATE_TABLE 			132
#define SQL_CREATE_TRANSLATION 			133
#define SQL_CREATE_VIEW 			134
#define SQL_DRIVER_HDESC 			135
#define SQL_DROP_ASSERTION 			136
#define SQL_DROP_CHARACTER_SET 			137
#define SQL_DROP_COLLATION 			138
#define SQL_DROP_DOMAIN 			139
#define SQL_DROP_SCHEMA 			140
#define SQL_DROP_TABLE 				141
#define SQL_DROP_TRANSLATION 			142
#define SQL_DROP_VIEW 				143
#define SQL_DYNAMIC_CURSOR_ATTRIBUTES1 		144
#define SQL_DYNAMIC_CURSOR_ATTRIBUTES2 		145
#define SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1 	146
#define SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2 	147
#define SQL_INDEX_KEYWORDS 			148
#define SQL_INFO_SCHEMA_VIEWS 			149
#define SQL_KEYSET_CURSOR_ATTRIBUTES1 		150
#define SQL_KEYSET_CURSOR_ATTRIBUTES2 		151
#define SQL_ODBC_INTERFACE_CONFORMANCE 		152
#define SQL_PARAM_ARRAY_ROW_COUNTS     		153
#define SQL_PARAM_ARRAY_SELECTS     		154
#define SQL_SCHEMA_TERM 			SQL_OWNER_TERM
#define SQL_SCHEMA_USAGE 			SQL_OWNER_USAGE
#define SQL_SQL92_DATETIME_FUNCTIONS 		155
#define SQL_SQL92_FOREIGN_KEY_DELETE_RULE 	156
#define SQL_SQL92_FOREIGN_KEY_UPDATE_RULE 	157
#define SQL_SQL92_GRANT 			158
#define SQL_SQL92_NUMERIC_VALUE_FUNCTIONS 	159
#define SQL_SQL92_PREDICATES 			160
#define SQL_SQL92_RELATIONAL_JOIN_OPERATORS 	161
#define SQL_SQL92_REVOKE 			162
#define SQL_SQL92_ROW_VALUE_CONSTRUCTOR 	163
#define SQL_SQL92_STRING_FUNCTIONS 		164
#define SQL_SQL92_VALUE_EXPRESSIONS 		165
#define SQL_STANDARD_CLI_CONFORMANCE 		166
#define SQL_STATIC_CURSOR_ATTRIBUTES1 		167
#define SQL_STATIC_CURSOR_ATTRIBUTES2 		168
#define SQL_AGGREGATE_FUNCTIONS 		169
#define SQL_DDL_INDEX 				170
#define SQL_DM_VER 				171
#define SQL_INSERT_STATEMENT 			172
#define SQL_UNION_STATEMENT 			SQL_UNION

/*
 *  New X/OPEN numbers
 */
#define SQL_XOPEN_CLI_YEAR               	10000
#define SQL_CURSOR_SENSITIVITY           	10001
#define SQL_DESCRIBE_PARAMETER           	10002
#define SQL_CATALOG_NAME                 	10003
#define SQL_COLLATION_SEQ                	10004
#define SQL_MAX_IDENTIFIER_LEN           	10005
#define SQL_MAXIMUM_IDENTIFIER_LENGTH 	 	SQL_MAX_IDENTIFIER_LEN

#define SQL_ASYNC_MODE 				10021	/* new X/Open spec */
#define SQL_MAX_ASYNC_CONCURRENT_STATEMENTS 	10022	/* new X/Open spec */
#endif  /* ODBCVER >= 0x0300 */

/*
 *  "SQL_CONVERT_" return value bitmasks
 */
#define SQL_CVT_CHAR 				0x00000001L
#define SQL_CVT_NUMERIC 			0x00000002L
#define SQL_CVT_DECIMAL 			0x00000004L
#define SQL_CVT_INTEGER 			0x00000008L
#define SQL_CVT_SMALLINT 			0x00000010L
#define SQL_CVT_FLOAT 				0x00000020L
#define SQL_CVT_REAL 				0x00000040L
#define SQL_CVT_DOUBLE 				0x00000080L
#define SQL_CVT_VARCHAR 			0x00000100L
#define SQL_CVT_LONGVARCHAR 			0x00000200L
#define SQL_CVT_BINARY 				0x00000400L
#define SQL_CVT_VARBINARY 			0x00000800L
#define SQL_CVT_BIT 				0x00001000L
#define SQL_CVT_TINYINT 			0x00002000L
#define SQL_CVT_BIGINT 				0x00004000L
#define SQL_CVT_DATE 				0x00008000L
#define SQL_CVT_TIME 				0x00010000L
#define SQL_CVT_TIMESTAMP 			0x00020000L
#define SQL_CVT_LONGVARBINARY 			0x00040000L

/*
 *  Conversion functions
 */
#define SQL_FN_CVT_CONVERT 			0x00000001L

/*
 *  String functions
 */
#define SQL_FN_STR_CONCAT 			0x00000001L
#define SQL_FN_STR_INSERT 			0x00000002L
#define SQL_FN_STR_LEFT 			0x00000004L
#define SQL_FN_STR_LTRIM 			0x00000008L
#define SQL_FN_STR_LENGTH 			0x00000010L
#define SQL_FN_STR_LOCATE 			0x00000020L
#define SQL_FN_STR_LCASE 			0x00000040L
#define SQL_FN_STR_REPEAT 			0x00000080L
#define SQL_FN_STR_REPLACE 			0x00000100L
#define SQL_FN_STR_RIGHT 			0x00000200L
#define SQL_FN_STR_RTRIM 			0x00000400L
#define SQL_FN_STR_SUBSTRING 			0x00000800L
#define SQL_FN_STR_UCASE 			0x00001000L
#define SQL_FN_STR_ASCII 			0x00002000L
#define SQL_FN_STR_CHAR 			0x00004000L
#define SQL_FN_STR_DIFFERENCE 			0x00008000L /* ODBC 2 */
#define SQL_FN_STR_LOCATE_2 			0x00010000L
#define SQL_FN_STR_SOUNDEX 			0x00020000L
#define SQL_FN_STR_SPACE 			0x00040000L

/*
 *  Numeric functions
 */
#define SQL_FN_NUM_ABS 				0x00000001L
#define SQL_FN_NUM_ACOS 			0x00000002L
#define SQL_FN_NUM_ASIN 			0x00000004L
#define SQL_FN_NUM_ATAN 			0x00000008L
#define SQL_FN_NUM_ATAN2 			0x00000010L
#define SQL_FN_NUM_CEILING 			0x00000020L
#define SQL_FN_NUM_COS 				0x00000040L
#define SQL_FN_NUM_COT 				0x00000080L
#define SQL_FN_NUM_EXP 				0x00000100L
#define SQL_FN_NUM_FLOOR 			0x00000200L
#define SQL_FN_NUM_LOG 				0x00000400L
#define SQL_FN_NUM_MOD 				0x00000800L
#define SQL_FN_NUM_SIGN 			0x00001000L
#define SQL_FN_NUM_SIN 				0x00002000L
#define SQL_FN_NUM_SQRT 			0x00004000L
#define SQL_FN_NUM_TAN 				0x00008000L
#define SQL_FN_NUM_PI 				0x00010000L
#define SQL_FN_NUM_RAND 			0x00020000L 	/* ODBC 2 */
#define SQL_FN_NUM_DEGREES 			0x00040000L
#define SQL_FN_NUM_LOG10 			0x00080000L
#define SQL_FN_NUM_POWER 			0x00100000L
#define SQL_FN_NUM_RADIANS 			0x00200000L
#define SQL_FN_NUM_ROUND 			0x00400000L
#define SQL_FN_NUM_TRUNCATE 			0x00800000L

/*
 *  Time/date functions
 */
#define SQL_FN_TD_NOW 				0x00000001L
#define SQL_FN_TD_CURDATE 			0x00000002L
#define SQL_FN_TD_DAYOFMONTH 			0x00000004L
#define SQL_FN_TD_DAYOFWEEK 			0x00000008L
#define SQL_FN_TD_DAYOFYEAR 			0x00000010L
#define SQL_FN_TD_MONTH 			0x00000020L
#define SQL_FN_TD_QUARTER 			0x00000040L
#define SQL_FN_TD_WEEK 				0x00000080L
#define SQL_FN_TD_YEAR 				0x00000100L
#define SQL_FN_TD_CURTIME 			0x00000200L
#define SQL_FN_TD_HOUR 				0x00000400L
#define SQL_FN_TD_MINUTE 			0x00000800L
#define SQL_FN_TD_SECOND 			0x00001000L
#define SQL_FN_TD_TIMESTAMPADD 			0x00002000L 	/* ODBC 2 */
#define SQL_FN_TD_TIMESTAMPDIFF 		0x00004000L
#define SQL_FN_TD_DAYNAME 			0x00008000L
#define SQL_FN_TD_MONTHNAME 			0x00010000L

/*
 *  System functions
 */
#define SQL_FN_SYS_USERNAME 			0x00000001L
#define SQL_FN_SYS_DBNAME 			0x00000002L
#define SQL_FN_SYS_IFNULL 			0x00000004L

/*
 *  SQL_TIMEDATE_ADD_INTERVALS and SQL_TIMEDATE_DIFF_INTERVALS functions
 */
#define SQL_FN_TSI_FRAC_SECOND 			0x00000001L
#define SQL_FN_TSI_SECOND 			0x00000002L
#define SQL_FN_TSI_MINUTE 			0x00000004L
#define SQL_FN_TSI_HOUR 			0x00000008L
#define SQL_FN_TSI_DAY 				0x00000010L
#define SQL_FN_TSI_WEEK 			0x00000020L
#define SQL_FN_TSI_MONTH 			0x00000040L
#define SQL_FN_TSI_QUARTER 			0x00000080L
#define SQL_FN_TSI_YEAR 			0x00000100L

/*
 *  ODBC API conformance
 */
#define SQL_OAC_NONE 				0x0000
#define SQL_OAC_LEVEL1 				0x0001
#define SQL_OAC_LEVEL2 				0x0002

/*
 *  SAG CLI conformance
 */
#define SQL_OSCC_NOT_COMPLIANT 			0x0000
#define SQL_OSCC_COMPLIANT 			0x0001

/*
 *  SQL Conformance 
 */
#define SQL_OSC_MINIMUM 			0x0000
#define SQL_OSC_CORE 				0x0001
#define SQL_OSC_EXTENDED 			0x0002

/*
 *  Concat NULL behaviour
 */
#define SQL_CB_NULL 				0x0000
#define SQL_CB_NON_NULL 			0x0001

/*
 *  Cursor commit/rollback behaviour
 */
#define SQL_CB_DELETE 				0x0000
#define SQL_CB_CLOSE 				0x0001
#define SQL_CB_PRESERVE 			0x0002

/*
 * Deprecated defines from prior versions of ODBC
 */
#define SQL_CC_DELETE 				SQL_CB_DELETE
#define SQL_CR_DELETE 				SQL_CB_DELETE
#define SQL_CC_CLOSE 				SQL_CB_CLOSE
#define SQL_CR_CLOSE 				SQL_CB_CLOSE
#define SQL_CC_PRESERVE 			SQL_CB_PRESERVE
#define SQL_CR_PRESERVE 			SQL_CB_PRESERVE

/*
 *  Identifier case
 */
#define SQL_IC_UPPER 				0x0001
#define SQL_IC_LOWER 				0x0002
#define SQL_IC_SENSITIVE 			0x0003
#define SQL_IC_MIXED 				0x0004

/*
 *  Transaction capabilities
 */
#define SQL_TC_NONE 				0x0000
#define SQL_TC_DML 				0x0001
#define SQL_TC_ALL 				0x0002
#define SQL_TC_DDL_COMMIT 			0x0003	/* ODBC 2 */
#define SQL_TC_DDL_IGNORE 			0x0004  /* ODBC 2 */

/*
 *  Scroll option masks
 */
#define SQL_SO_FORWARD_ONLY 			0x00000001L
#define SQL_SO_KEYSET_DRIVEN 			0x00000002L
#define SQL_SO_DYNAMIC 				0x00000004L
#define SQL_SO_MIXED 				0x00000008L
#define SQL_SO_STATIC 				0x00000010L 	/* ODBC 2 */

/*
 *  Scroll concurrency option masks
 */
#define SQL_SCCO_READ_ONLY 			0x00000001L
#define SQL_SCCO_LOCK 				0x00000002L
#define SQL_SCCO_OPT_ROWVER 			0x00000004L
#define SQL_SCCO_OPT_VALUES 			0x00000008L
/* Deprecated define from ODBC 1.0 */
#define SQL_SCCO_OPT_TIMESTAMP  		SQL_SCCO_OPT_ROWVER

/*
 *  Fetch direction option masks
 */
#define SQL_FD_FETCH_NEXT 			0x00000001L
#define SQL_FD_FETCH_FIRST 			0x00000002L
#define SQL_FD_FETCH_LAST 			0x00000004L
#define SQL_FD_FETCH_PREV 			0x00000008L
#define SQL_FD_FETCH_ABSOLUTE 			0x00000010L
#define SQL_FD_FETCH_RELATIVE 			0x00000020L
#define SQL_FD_FETCH_RESUME 			0x00000040L
#define SQL_FD_FETCH_BOOKMARK 			0x00000080L 	/* ODBC 2 */
#define SQL_FD_FETCH_PRIOR 			SQL_FD_FETCH_PREV

/*
 *  Transaction isolation option masks
 */
#define SQL_TXN_READ_UNCOMMITTED 		0x00000001L
#define SQL_TXN_READ_COMMITTED 			0x00000002L
#define SQL_TXN_REPEATABLE_READ 		0x00000004L
#define SQL_TXN_SERIALIZABLE 			0x00000008L
#if (ODBCVER < 0x0300)
#define SQL_TXN_VERSIONING 			0x00000010L
#endif

/*
 *  Correlation names
 */
#define SQL_CN_NONE 				0x0000
#define SQL_CN_DIFFERENT 			0x0001
#define SQL_CN_ANY 				0x0002

/*
 *  Non nullable columns
 */
#define SQL_NNC_NULL 			   	0x0000
#define SQL_NNC_NON_NULL 			0x0001

/*
 *  Collation order for NULL fields
 */
#define SQL_NC_HIGH 				0x0000
#define SQL_NC_LOW 				0x0001
#define SQL_NC_START 				0x0002
#define SQL_NC_END 				0x0004

/*
 *  File usage
 */
#define SQL_FILE_NOT_SUPPORTED 			0x0000
#define SQL_FILE_TABLE 				0x0001
#define SQL_FILE_QUALIFIER 			0x0002

/*
 *  GETDATA extentions
 */
#define SQL_GD_ANY_COLUMN 			0x00000001L
#define SQL_GD_ANY_ORDER 			0x00000002L
#define SQL_GD_BLOCK 				0x00000004L
#define SQL_GD_BOUND 				0x00000008L

/*
 *  Alter table options
 */
#define SQL_AT_ADD_COLUMN 			0x00000001L
#define SQL_AT_DROP_COLUMN 			0x00000002L

/*
 *  Positioned_statements masks 
 */
#define SQL_PS_POSITIONED_DELETE 		0x00000001L
#define SQL_PS_POSITIONED_UPDATE 		0x00000002L
#define SQL_PS_SELECT_FOR_UPDATE 		0x00000004L

/*
 *  GROUP BY masks
 */
#define SQL_GB_NOT_SUPPORTED 			0x0000
#define SQL_GB_GROUP_BY_EQUALS_SELECT 		0x0001
#define SQL_GB_GROUP_BY_CONTAINS_SELECT 	0x0002
#define SQL_GB_NO_RELATION 			0x0003

/*
 *  Owner usage masks
 */
#define SQL_OU_DML_STATEMENTS 			0x00000001L
#define SQL_OU_PROCEDURE_INVOCATION 		0x00000002L
#define SQL_OU_TABLE_DEFINITION 		0x00000004L
#define SQL_OU_INDEX_DEFINITION 		0x00000008L
#define SQL_OU_PRIVILEGE_DEFINITION 		0x00000010L

/*
 *  Qualifier usage mask
 */
#define SQL_QU_DML_STATEMENTS 			0x00000001L
#define SQL_QU_PROCEDURE_INVOCATION 		0x00000002L
#define SQL_QU_TABLE_DEFINITION 		0x00000004L
#define SQL_QU_INDEX_DEFINITION 		0x00000008L
#define SQL_QU_PRIVILEGE_DEFINITION 		0x00000010L

/*
 *  Subqueries masks
 */
#define SQL_SQ_COMPARISON 			0x00000001L
#define SQL_SQ_EXISTS 				0x00000002L
#define SQL_SQ_IN 				0x00000004L
#define SQL_SQ_QUANTIFIED 			0x00000008L
#define SQL_SQ_CORRELATED_SUBQUERIES 		0x00000010L

/*
 *  Union masks
 */
#define SQL_U_UNION 				0x00000001L
#define SQL_U_UNION_ALL 			0x00000002L

/* 
 *  SQL_STATIC_SENSITIVITY values 
 */
#define SQL_SS_ADDITIONS 			0x00000001L
#define SQL_SS_DELETIONS 			0x00000002L
#define SQL_SS_UPDATES 				0x00000004L

/* 
 *  SQL_LOCK_TYPESL masks 
 */
#define SQL_LCK_NO_CHANGE 			0x00000001L
#define SQL_LCK_EXCLUSIVE 			0x00000002L
#define SQL_LCK_UNLOCK 				0x00000004L

/*
 *  Qualifier location masks
 */
#define SQL_QL_START 				0x0001L
#define SQL_QL_END 				0x0002L

/*
 *  Outerjoin capabilities
 */
#define SQL_OJ_LEFT 				0x00000001L 
#define SQL_OJ_RIGHT 				0x00000002L
#define SQL_OJ_FULL 				0x00000004L
#define SQL_OJ_NESTED 				0x00000008L
#define SQL_OJ_NOT_ORDERED 			0x00000010L
#define SQL_OJ_INNER 				0x00000020L
#define SQL_OJ_ALL_COMPARISON_OPS 		0x00000040L


/*
 *  ODBC 3.0 GetInfo values and bitmasks
 */
#if (ODBCVER >= 0x0300)

/*
 *  SQL_AGGREGATE_FUNCTIONS bitmasks 
 */
#define SQL_AF_AVG 				0x00000001L
#define SQL_AF_COUNT 				0x00000002L
#define SQL_AF_MAX 				0x00000004L
#define SQL_AF_MIN 				0x00000008L
#define SQL_AF_SUM 				0x00000010L
#define SQL_AF_DISTINCT 			0x00000020L

#define SQL_AF_ALL 				(SQL_AF_AVG 	| \
						 SQL_AF_COUNT 	| \
						 SQL_AF_MAX 	| \
						 SQL_AF_MIN 	| \
						 SQL_AF_SUM 	| \
						 SQL_AF_DISTINCT)

/*
 *  SQL_SQL_CONFORMANCE bitmasks 
 */
#define SQL_SC_SQL92_ENTRY 			0x00000001L
#define SQL_SC_FIPS127_2_TRANSITIONAL 		0x00000002L
#define SQL_SC_SQL92_INTERMEDIATE 		0x00000004L
#define SQL_SC_SQL92_FULL 			0x00000008L

/*
 *  SQL_DATETIME_LITERALS bitmasks 
 */
#define SQL_DL_SQL92_DATE 			0x00000001L
#define SQL_DL_SQL92_TIME 			0x00000002L
#define SQL_DL_SQL92_TIMESTAMP 			0x00000004L
#define SQL_DL_SQL92_INTERVAL_YEAR 		0x00000008L
#define SQL_DL_SQL92_INTERVAL_MONTH 		0x00000010L
#define SQL_DL_SQL92_INTERVAL_DAY 		0x00000020L
#define SQL_DL_SQL92_INTERVAL_HOUR 		0x00000040L
#define SQL_DL_SQL92_INTERVAL_MINUTE 		0x00000080L
#define SQL_DL_SQL92_INTERVAL_SECOND 		0x00000100L
#define SQL_DL_SQL92_INTERVAL_YEAR_TO_MONTH 	0x00000200L
#define SQL_DL_SQL92_INTERVAL_DAY_TO_HOUR 	0x00000400L
#define SQL_DL_SQL92_INTERVAL_DAY_TO_MINUTE 	0x00000800L
#define SQL_DL_SQL92_INTERVAL_DAY_TO_SECOND 	0x00001000L
#define SQL_DL_SQL92_INTERVAL_HOUR_TO_MINUTE 	0x00002000L
#define SQL_DL_SQL92_INTERVAL_HOUR_TO_SECOND 	0x00004000L
#define SQL_DL_SQL92_INTERVAL_MINUTE_TO_SECOND 	0x00008000L

/*
 *  SQL_CATALOG_LOCATION values 
 */
#define SQL_CL_START 				SQL_QL_START
#define SQL_CL_END 				SQL_QL_END

/*
 *  SQL_BATCH_ROW_COUNT values
 */
#define SQL_BRC_PROCEDURES 			0x0000001
#define SQL_BRC_EXPLICIT 			0x0000002
#define SQL_BRC_ROLLED_UP 			0x0000004

/*
 *  SQL_BATCH_SUPPORT bitmasks
 */
#define SQL_BS_SELECT_EXPLICIT 			0x00000001L
#define SQL_BS_ROW_COUNT_EXPLICIT 		0x00000002L
#define SQL_BS_SELECT_PROC 			0x00000004L
#define SQL_BS_ROW_COUNT_PROC 			0x00000008L

/*
 *  SQL_PARAM_ARRAY_ROW_COUNTS values
 */
#define SQL_PARC_BATCH 				1
#define SQL_PARC_NO_BATCH 			2

/*
 *  SQL_PARAM_ARRAY_SELECT_BATCH values
 */
#define SQL_PAS_BATCH 				1
#define SQL_PAS_NO_BATCH 			2
#define SQL_PAS_NO_SELECT 			3

/*
 *  SQL_INDEX_KEYWORDS bitmasks
 */
#define SQL_IK_NONE 				0x00000000L
#define SQL_IK_ASC 				0x00000001L
#define SQL_IK_DESC 				0x00000002L
#define SQL_IK_ALL 				(SQL_IK_ASC | SQL_IK_DESC)

/*
 *  SQL_INFO_SCHEMA_VIEWS bitmasks
 */
#define SQL_ISV_ASSERTIONS 			0x00000001L
#define SQL_ISV_CHARACTER_SETS 			0x00000002L
#define SQL_ISV_CHECK_CONSTRAINTS 		0x00000004L
#define SQL_ISV_COLLATIONS 			0x00000008L
#define SQL_ISV_COLUMN_DOMAIN_USAGE 		0x00000010L
#define SQL_ISV_COLUMN_PRIVILEGES 		0x00000020L
#define SQL_ISV_COLUMNS 			0x00000040L
#define SQL_ISV_CONSTRAINT_COLUMN_USAGE 	0x00000080L
#define SQL_ISV_CONSTRAINT_TABLE_USAGE 		0x00000100L
#define SQL_ISV_DOMAIN_CONSTRAINTS 		0x00000200L
#define SQL_ISV_DOMAINS 			0x00000400L
#define SQL_ISV_KEY_COLUMN_USAGE 		0x00000800L
#define SQL_ISV_REFERENTIAL_CONSTRAINTS 	0x00001000L
#define SQL_ISV_SCHEMATA 			0x00002000L
#define SQL_ISV_SQL_LANGUAGES 			0x00004000L
#define SQL_ISV_TABLE_CONSTRAINTS 		0x00008000L
#define SQL_ISV_TABLE_PRIVILEGES 		0x00010000L
#define SQL_ISV_TABLES 				0x00020000L
#define SQL_ISV_TRANSLATIONS 			0x00040000L
#define SQL_ISV_USAGE_PRIVILEGES 		0x00080000L
#define SQL_ISV_VIEW_COLUMN_USAGE 		0x00100000L
#define SQL_ISV_VIEW_TABLE_USAGE 		0x00200000L
#define SQL_ISV_VIEWS 				0x00400000L

/*
 *  SQL_ASYNC_MODE values
 */
#define SQL_AM_NONE 				0
#define SQL_AM_CONNECTION 			1
#define SQL_AM_STATEMENT 			2

/*
 *  SQL_ALTER_DOMAIN bitmasks
 */
#define SQL_AD_CONSTRAINT_NAME_DEFINITION 		0x00000001L
#define SQL_AD_ADD_DOMAIN_CONSTRAINT 	 		0x00000002L
#define SQL_AD_DROP_DOMAIN_CONSTRAINT 	 		0x00000004L
#define SQL_AD_ADD_DOMAIN_DEFAULT   	 		0x00000008L
#define SQL_AD_DROP_DOMAIN_DEFAULT   	 		0x00000010L
#define SQL_AD_ADD_CONSTRAINT_INITIALLY_DEFERRE 	0x00000020L
#define SQL_AD_ADD_CONSTRAINT_INITIALLY_IMMEDIATE 	0x00000040L
#define SQL_AD_ADD_CONSTRAINT_DEFERRABLE 		0x00000080L
#define SQL_AD_ADD_CONSTRAINT_NON_DEFERRABLE 		0x00000100L


/*
 *  SQL_CREATE_SCHEMA bitmasks 
 */
#define SQL_CS_CREATE_SCHEMA 			0x00000001L
#define SQL_CS_AUTHORIZATION 			0x00000002L
#define SQL_CS_DEFAULT_CHARACTER_SET 		0x00000004L

/*
 *  SQL_CREATE_TRANSLATION bitmasks 
 */
#define SQL_CTR_CREATE_TRANSLATION 		0x00000001L

/*
 *  SQL_CREATE_ASSERTION bitmasks 
 */
#define SQL_CA_CREATE_ASSERTION 		0x00000001L
#define SQL_CA_CONSTRAINT_INITIALLY_DEFERRED 	0x00000010L
#define SQL_CA_CONSTRAINT_INITIALLY_IMMEDIATE 	0x00000020L
#define SQL_CA_CONSTRAINT_DEFERRABLE 		0x00000040L
#define SQL_CA_CONSTRAINT_NON_DEFERRABLE 	0x00000080L

/*
 *  SQL_CREATE_CHARACTER_SET bitmasks 
 */
#define SQL_CCS_CREATE_CHARACTER_SET 		0x00000001L
#define SQL_CCS_COLLATE_CLAUSE 			0x00000002L
#define SQL_CCS_LIMITED_COLLATION 		0x00000004L

/*
 *  SQL_CREATE_COLLATION bitmasks 
 */
#define SQL_CCOL_CREATE_COLLATION 		0x00000001L

/*
 *  SQL_CREATE_DOMAIN bitmasks 
 */
#define SQL_CDO_CREATE_DOMAIN 			0x00000001L
#define SQL_CDO_DEFAULT 			0x00000002L
#define SQL_CDO_CONSTRAINT 			0x00000004L
#define SQL_CDO_COLLATION 			0x00000008L
#define SQL_CDO_CONSTRAINT_NAME_DEFINITION 	0x00000010L
#define SQL_CDO_CONSTRAINT_INITIALLY_DEFERRED 	0x00000020L
#define SQL_CDO_CONSTRAINT_INITIALLY_IMMEDIATE 	0x00000040L
#define SQL_CDO_CONSTRAINT_DEFERRABLE 		0x00000080L
#define SQL_CDO_CONSTRAINT_NON_DEFERRABLE 	0x00000100L

/*
 *  SQL_CREATE_TABLE bitmasks 
 */
#define SQL_CT_CREATE_TABLE 			0x00000001L
#define SQL_CT_COMMIT_PRESERVE 			0x00000002L
#define SQL_CT_COMMIT_DELETE 			0x00000004L
#define SQL_CT_GLOBAL_TEMPORARY 		0x00000008L
#define SQL_CT_LOCAL_TEMPORARY 			0x00000010L
#define SQL_CT_CONSTRAINT_INITIALLY_DEFERRED 	0x00000020L
#define SQL_CT_CONSTRAINT_INITIALLY_IMMEDIATE 	0x00000040L
#define SQL_CT_CONSTRAINT_DEFERRABLE 		0x00000080L
#define SQL_CT_CONSTRAINT_NON_DEFERRABLE 	0x00000100L
#define SQL_CT_COLUMN_CONSTRAINT 		0x00000200L
#define SQL_CT_COLUMN_DEFAULT 			0x00000400L
#define SQL_CT_COLUMN_COLLATION 		0x00000800L
#define SQL_CT_TABLE_CONSTRAINT 		0x00001000L
#define SQL_CT_CONSTRAINT_NAME_DEFINITION 	0x00002000L

/*
 *  SQL_DDL_INDEX bitmasks 
 */
#define SQL_DI_CREATE_INDEX 			0x00000001L
#define SQL_DI_DROP_INDEX 			0x00000002L

/*
 *  SQL_DROP_COLLATION bitmasks 
 */
#define SQL_DC_DROP_COLLATION 			0x00000001L

/*
 *  SQL_DROP_DOMAIN bitmasks 
 */
#define SQL_DD_DROP_DOMAIN 			0x00000001L
#define SQL_DD_RESTRICT 			0x00000002L
#define SQL_DD_CASCADE 				0x00000004L

/*
 *  SQL_DROP_SCHEMA bitmasks 
 */
#define SQL_DS_DROP_SCHEMA 			0x00000001L
#define SQL_DS_RESTRICT 			0x00000002L
#define SQL_DS_CASCADE 				0x00000004L

/*
 *  SQL_DROP_CHARACTER_SET bitmasks 
 */
#define SQL_DCS_DROP_CHARACTER_SET 		0x00000001L

/*
 *  SQL_DROP_ASSERTION bitmasks 
 */
#define SQL_DA_DROP_ASSERTION 			0x00000001L

/*
 *  SQL_DROP_TABLE bitmasks 
 */
#define SQL_DT_DROP_TABLE 			0x00000001L
#define SQL_DT_RESTRICT 			0x00000002L
#define SQL_DT_CASCADE 				0x00000004L

/*
 *  SQL_DROP_TRANSLATION bitmasks 
 */
#define SQL_DTR_DROP_TRANSLATION 		0x00000001L

/*
 *  SQL_DROP_VIEW bitmasks 
 */
#define SQL_DV_DROP_VIEW 			0x00000001L
#define SQL_DV_RESTRICT 			0x00000002L
#define SQL_DV_CASCADE 				0x00000004L

/*
 *  SQL_INSERT_STATEMENT bitmasks 
 */
#define SQL_IS_INSERT_LITERALS 			0x00000001L
#define SQL_IS_INSERT_SEARCHED 			0x00000002L
#define SQL_IS_SELECT_INTO 			0x00000004L

/*
 *  SQL_ODBC_INTERFACE_CONFORMANCE values 
 */
#define SQL_OIC_CORE 				1UL
#define SQL_OIC_LEVEL1 				2UL
#define SQL_OIC_LEVEL2 				3UL

/*
 *  SQL_SQL92_FOREIGN_KEY_DELETE_RULE bitmasks 
 */
#define SQL_SFKD_CASCADE 			0x00000001L
#define SQL_SFKD_NO_ACTION 			0x00000002L
#define SQL_SFKD_SET_DEFAULT 			0x00000004L
#define SQL_SFKD_SET_NULL 			0x00000008L

/*
 *  SQL_SQL92_FOREIGN_KEY_UPDATE_RULE bitmasks 
 */
#define SQL_SFKU_CASCADE 			0x00000001L
#define SQL_SFKU_NO_ACTION 			0x00000002L
#define SQL_SFKU_SET_DEFAULT 			0x00000004L
#define SQL_SFKU_SET_NULL 			0x00000008L

/*
 *  SQL_SQL92_GRANT bitmasks 
 */
#define SQL_SG_USAGE_ON_DOMAIN 			0x00000001L
#define SQL_SG_USAGE_ON_CHARACTER_SET 		0x00000002L
#define SQL_SG_USAGE_ON_COLLATION 		0x00000004L
#define SQL_SG_USAGE_ON_TRANSLATION 		0x00000008L
#define SQL_SG_WITH_GRANT_OPTION 		0x00000010L
#define SQL_SG_DELETE_TABLE 			0x00000020L
#define SQL_SG_INSERT_TABLE 			0x00000040L
#define SQL_SG_INSERT_COLUMN 			0x00000080L
#define SQL_SG_REFERENCES_TABLE 		0x00000100L
#define SQL_SG_REFERENCES_COLUMN 		0x00000200L
#define SQL_SG_SELECT_TABLE 			0x00000400L
#define SQL_SG_UPDATE_TABLE 			0x00000800L
#define SQL_SG_UPDATE_COLUMN 			0x00001000L

/*
 *  SQL_SQL92_PREDICATES bitmasks 
 */
#define SQL_SP_EXISTS 				0x00000001L
#define SQL_SP_ISNOTNULL 			0x00000002L
#define SQL_SP_ISNULL 				0x00000004L
#define SQL_SP_MATCH_FULL 			0x00000008L
#define SQL_SP_MATCH_PARTIAL 			0x00000010L
#define SQL_SP_MATCH_UNIQUE_FULL 		0x00000020L
#define SQL_SP_MATCH_UNIQUE_PARTIAL 		0x00000040L
#define SQL_SP_OVERLAPS 			0x00000080L
#define SQL_SP_UNIQUE 				0x00000100L
#define SQL_SP_LIKE 				0x00000200L
#define SQL_SP_IN 				0x00000400L
#define SQL_SP_BETWEEN 				0x00000800L
#define SQL_SP_COMPARISON 			0x00001000L
#define SQL_SP_QUANTIFIED_COMPARISON 		0x00002000L

/*
 *  SQL_SQL92_RELATIONAL_JOIN_OPERATORS bitmasks 
 */
#define SQL_SRJO_CORRESPONDING_CLAUSE 		0x00000001L
#define SQL_SRJO_CROSS_JOIN 			0x00000002L
#define SQL_SRJO_EXCEPT_JOIN 			0x00000004L
#define SQL_SRJO_FULL_OUTER_JOIN 		0x00000008L
#define SQL_SRJO_INNER_JOIN 			0x00000010L
#define SQL_SRJO_INTERSECT_JOIN 		0x00000020L
#define SQL_SRJO_LEFT_OUTER_JOIN 		0x00000040L
#define SQL_SRJO_NATURAL_JOIN 			0x00000080L
#define SQL_SRJO_RIGHT_OUTER_JOIN 		0x00000100L
#define SQL_SRJO_UNION_JOIN 			0x00000200L

/*
 *  SQL_SQL92_REVOKE bitmasks 
 */
#define SQL_SR_USAGE_ON_DOMAIN 			0x00000001L
#define SQL_SR_USAGE_ON_CHARACTER_SET 		0x00000002L
#define SQL_SR_USAGE_ON_COLLATION 		0x00000004L
#define SQL_SR_USAGE_ON_TRANSLATION 		0x00000008L
#define SQL_SR_GRANT_OPTION_FOR 		0x00000010L
#define SQL_SR_CASCADE 				0x00000020L
#define SQL_SR_RESTRICT 			0x00000040L
#define SQL_SR_DELETE_TABLE 			0x00000080L
#define SQL_SR_INSERT_TABLE 			0x00000100L
#define SQL_SR_INSERT_COLUMN 			0x00000200L
#define SQL_SR_REFERENCES_TABLE 		0x00000400L
#define SQL_SR_REFERENCES_COLUMN 		0x00000800L
#define SQL_SR_SELECT_TABLE 			0x00001000L
#define SQL_SR_UPDATE_TABLE 			0x00002000L
#define SQL_SR_UPDATE_COLUMN 			0x00004000L

/*
 *  SQL_SQL92_ROW_VALUE_CONSTRUCTOR bitmasks 
 */
#define SQL_SRVC_VALUE_EXPRESSION 		0x00000001L
#define SQL_SRVC_NULL 				0x00000002L
#define SQL_SRVC_DEFAULT 			0x00000004L
#define SQL_SRVC_ROW_SUBQUERY 			0x00000008L

/*
 *  SQL_SQL92_VALUE_EXPRESSIONS bitmasks 
 */
#define SQL_SVE_CASE 				0x00000001L
#define SQL_SVE_CAST 				0x00000002L
#define SQL_SVE_COALESCE 			0x00000004L
#define SQL_SVE_NULLIF 				0x00000008L

/*
 *  SQL_STANDARD_CLI_CONFORMANCE bitmasks 
 */
#define SQL_SCC_XOPEN_CLI_VERSION1 		0x00000001L
#define SQL_SCC_ISO92_CLI 			0x00000002L

/*
 *  SQL_UNION_STATEMENT bitmasks 
 */
#define SQL_US_UNION 				SQL_U_UNION
#define SQL_US_UNION_ALL 			SQL_U_UNION_ALL

#endif /* ODBCVER >= 0x0300 */

/*
 *  Options for SQLGetStmtOption/SQLSetStmtOption
 */
#define SQL_QUERY_TIMEOUT 			0
#define SQL_MAX_ROWS 				1
#define SQL_NOSCAN 				2
#define SQL_MAX_LENGTH 				3
#define SQL_ASYNC_ENABLE 			4
#define SQL_BIND_TYPE 				5
#define SQL_CURSOR_TYPE 			6	/* ODBC 2 */
#define SQL_CONCURRENCY 			7
#define SQL_KEYSET_SIZE 			8
#define SQL_ROWSET_SIZE 			9
#define SQL_SIMULATE_CURSOR 			10
#define SQL_RETRIEVE_DATA 			11
#define SQL_USE_BOOKMARKS 			12
#define SQL_GET_BOOKMARK 			13	/* GetStmtOption Only */
#define SQL_ROW_NUMBER 				14	/* GetStmtOption Only */

#define SQL_STMT_OPT_MIN 			SQL_QUERY_TIMEOUT
#define SQL_STMT_OPT_MAX 			SQL_ROW_NUMBER

/*
 *  SQL_QUERY_TIMEOUT options 
 */
#define SQL_QUERY_TIMEOUT_DEFAULT 		0UL

/* 
 *  SQL_MAX_ROWS options 
 */
#define SQL_MAX_ROWS_DEFAULT 			0UL

/*
 *  SQL_NOSCAN options 
 */
#define SQL_NOSCAN_OFF 				0UL     /* 1.0 FALSE */
#define SQL_NOSCAN_ON 				1UL     /* 1.0 TRUE */
#define SQL_NOSCAN_DEFAULT 			SQL_NOSCAN_OFF

/*
 *  SQL_MAX_LENGTH options 
 */
#define SQL_MAX_LENGTH_DEFAULT 			0UL

/*
 *  SQL_ASYNC_ENABLE options 
 */
#define SQL_ASYNC_ENABLE_OFF 			0UL
#define SQL_ASYNC_ENABLE_ON 			1UL
#define SQL_ASYNC_ENABLE_DEFAULT 		SQL_ASYNC_ENABLE_OFF

/* 
 *  SQL_BIND_TYPE options 
 */
#define SQL_BIND_BY_COLUMN 			0UL
#define SQL_BIND_TYPE_DEFAULT 			SQL_BIND_BY_COLUMN  

/*
 *  Options for SQLSetConnectOption/SQLGetConnectOption
 */
#define SQL_ACCESS_MODE 			101
#define SQL_AUTOCOMMIT 				102
#define SQL_LOGIN_TIMEOUT 			103
#define SQL_OPT_TRACE 				104
#define SQL_OPT_TRACEFILE 			105
#define SQL_TRANSLATE_DLL 			106
#define SQL_TRANSLATE_OPTION 			107
#define SQL_TXN_ISOLATION 			108
#define SQL_CURRENT_QUALIFIER 			109
#define SQL_ODBC_CURSORS 			110
#define SQL_QUIET_MODE 				111
#define SQL_PACKET_SIZE 			112

#define SQL_CONN_OPT_MIN 			SQL_ACCESS_MODE
#define SQL_CONN_OPT_MAX 			SQL_PACKET_SIZE
#define SQL_CONNECT_OPT_DRVR_START 		1000

/*
 *  SQL_LOGIN_TIMEOUT options
 */
#define SQL_LOGIN_TIMEOUT_DEFAULT 		15UL

/*
 *  SQL_OPT_TRACE options 
 */
#define SQL_OPT_TRACE_OFF 			0UL
#define SQL_OPT_TRACE_ON 			1UL
#define SQL_OPT_TRACE_DEFAULT 			SQL_OPT_TRACE_OFF
#ifdef unix
#define SQL_OPT_TRACE_FILE_DEFAULT 		"/tmp/sql.log"
#else
#define SQL_OPT_TRACE_FILE_DEFAULT 		"\\SQL.LOG"
#endif

/*
 *  SQLExtendedFetch "fFetchType" values
 */
#define SQL_FETCH_NEXT 				1
#define SQL_FETCH_FIRST 			2
#define SQL_FETCH_LAST 				3
#define SQL_FETCH_PREV 				4
#define SQL_FETCH_ABSOLUTE 			5
#define SQL_FETCH_RELATIVE 			6
#define SQL_FETCH_RESUME 			7
#define SQL_FETCH_BOOKMARK     			8
#define SQL_FETCH_PRIOR 				SQL_FETCH_PREV

/*
 *  SQLExtendedFetch "rgfRowStatus" element values
 */
#define SQL_ROW_SUCCESS 			0
#define SQL_ROW_DELETED 			1
#define SQL_ROW_UPDATED 			2
#define SQL_ROW_NOROW 				3
#define SQL_ROW_ADDED 				4
#define SQL_ROW_ERROR 				5

/*
 *  Defines for SQLForeignKeys (returned in result set)
 */
#define SQL_CASCADE 				0
#define SQL_RESTRICT 				1
#define SQL_SET_NULL 				2
#define SQL_NO_ACTION 				3
#define SQL_SET_DEFAULT 			4

/*
 *  Defines for SQLProcedureColumns (returned in the result set)
 *  and SQLBindParameter
 */
#define SQL_PARAM_TYPE_UNKNOWN 			0
#define SQL_PARAM_INPUT 			1
#define SQL_PARAM_INPUT_OUTPUT 			2
#define SQL_RESULT_COL 				3
#define SQL_PARAM_OUTPUT 			4
#define SQL_RETURN_VALUE 			5

/*
 *  Defines for SQLProcedures (returned in the result set)
 */
#define SQL_PT_UNKNOWN 				0
#define SQL_PT_PROCEDURE 			1
#define SQL_PT_FUNCTION 			2


/* 
 *  Defines used by Driver Manager when mapping SQLSetParam to SQLBindParameter
 */
#define SQL_PARAM_TYPE_DEFAULT 			SQL_PARAM_INPUT_OUTPUT
#define SQL_SETPARAM_VALUE_MAX 			(-1L)	

/*
 *  Defines for SQLSetScrollOptions
 */
#define SQL_CONCUR_READ_ONLY 			1
#define SQL_CONCUR_LOCK 			2
#define SQL_CONCUR_ROWVER 			3
#define SQL_CONCUR_VALUES 			4
#define SQL_CONCUR_DEFAULT     			SQL_CONCUR_READ_ONLY

/*
 *  Deprecated define from ODBC 1.0 
 */
#define SQL_CONCUR_TIMESTAMP 			SQL_CONCUR_ROWVER
#define SQL_SCROLL_FORWARD_ONLY 		0L
#define SQL_SCROLL_KEYSET_DRIVEN 		(-1L)
#define SQL_SCROLL_DYNAMIC 			(-2L)
#define SQL_SCROLL_STATIC 			(-3L)

/*
 *  Defines for SQLStatistics
 */
#define SQL_INDEX_UNIQUE 			0
#define SQL_INDEX_ALL 				1

#define SQL_ENSURE 				1
#define SQL_QUICK 				0

/*
 *  Defines for SQLStatistics (returned in the result set)
 */
#define SQL_TABLE_STAT 				0
#define SQL_INDEX_CLUSTERED 			1
#define SQL_INDEX_HASHED 			2
#define SQL_INDEX_OTHER 			3

/*
 *  Defines for SQLSpecialColumns
 */
#define SQL_PC_UNKNOWN 				0
#define SQL_PC_NOT_PSEUDO 			1
#define SQL_PC_PSEUDO 				2
#define SQL_PC_NON_PSEUDO 			SQL_PC_NOT_PSEUDO

/*
 *  Defines for SQLSetPos
 */
#define SQL_ENTIRE_ROWSET 			0

/*
 *  Operations in SQLSetPos
 */
#define SQL_POSITION 				0
#define SQL_REFRESH 				1
#define SQL_UPDATE 				2
#define SQL_DELETE 				3
#define SQL_ADD 				4

/*
 * Flag for SQLSetPos
 */
#define SQL_POS_POSITION 			0x00000001L
#define SQL_POS_REFRESH 			0x00000002L
#define SQL_POS_UPDATE 				0x00000004L
#define SQL_POS_DELETE 				0x00000008L
#define SQL_POS_ADD 				0x00000010L

/*
 *  SQLBulkOperations
 */
#define SQL_ADD 				4
#if (ODBCVER >= 0x0300)
#define SQL_UPATE_BY_BOOKMARK 			5
#define SQL_DELETE_BY_BOOKMARK 			6
#define SQL_FETCH_BY_BOOKMARK 			7
#endif /* ODBCVER >= 0x0300 */

/*
 *  Lock options in SQLSetPos
 */
#define SQL_LOCK_NO_CHANGE 			0
#define SQL_LOCK_EXCLUSIVE 			1
#define SQL_LOCK_UNLOCK 			2

/*
 *  Macros for SQLSetPos
 */
#define SQL_POSITION_TO(hstmt,irow) SQLSetPos(hstmt,irow,SQL_POSITION,SQL_LOCK_NO_CHANGE)
#define SQL_LOCK_RECORD(hstmt,irow,fLock) SQLSetPos(hstmt,irow,SQL_POSITION,fLock)
#define SQL_REFRESH_RECORD(hstmt,irow,fLock) SQLSetPos(hstmt,irow,SQL_REFRESH,fLock)
#define SQL_UPDATE_RECORD(hstmt,irow) SQLSetPos(hstmt,irow,SQL_UPDATE,SQL_LOCK_NO_CHANGE)
#define SQL_DELETE_RECORD(hstmt,irow) SQLSetPos(hstmt,irow,SQL_DELETE,SQL_LOCK_NO_CHANGE)
#define SQL_ADD_RECORD(hstmt,irow) SQLSetPos(hstmt,irow,SQL_ADD,SQL_LOCK_NO_CHANGE)

/*
 *  SQL_ODBC_CURSORS options 
 */
#define SQL_CUR_USE_IF_NEEDED 			0UL
#define SQL_CUR_USE_ODBC 			1UL
#define SQL_CUR_USE_DRIVER 			2UL
#define SQL_CUR_DEFAULT 			SQL_CUR_USE_DRIVER

/*
 *  SQL_CURSOR_TYPE options
 */
#define SQL_CURSOR_FORWARD_ONLY        		0UL
#define SQL_CURSOR_KEYSET_DRIVEN        	1UL
#define SQL_CURSOR_DYNAMIC              	2UL
#define SQL_CURSOR_STATIC               	3UL
#define SQL_CURSOR_TYPE_DEFAULT         	SQL_CURSOR_FORWARD_ONLY 

/*
 *  SQL_KEYSET_SIZE options
 */
#define SQL_KEYSET_SIZE_DEFAULT         	0UL

/*
 *  SQL_ROWSET_SIZE options
 */
#define SQL_ROWSET_SIZE_DEFAULT         	1UL

/*
 *  SQL_SIMULATE_CURSOR options
 */
#define SQL_SC_NON_UNIQUE               	0UL
#define SQL_SC_TRY_UNIQUE              		1UL
#define SQL_SC_UNIQUE                   	2UL

/*
 *  SQL_RETRIEVE_DATA options
 */
#define SQL_RD_OFF                      	0UL
#define SQL_RD_ON                       	1UL
#define SQL_RD_DEFAULT                  	SQL_RD_ON

/*
 *  SQL_USE_BOOKMARKS options
 */
#define SQL_UB_OFF              		0UL
#define SQL_UB_ON               		1UL
#define SQL_UB_DEFAULT          		SQL_UB_OFF

/*
 *  SQL_BOOKMARK_PERSISTENCE values
 */
#define SQL_BP_CLOSE            		0x00000001L
#define SQL_BP_DELETE           		0x00000002L
#define SQL_BP_DROP             		0x00000004L
#define SQL_BP_TRANSACTION      		0x00000008L
#define SQL_BP_UPDATE           		0x00000010L
#define SQL_BP_OTHER_HSTMT      		0x00000020L
#define SQL_BP_SCROLL           		0x00000040L


/*
 *  Column types and scopes in SQLSpecialColumns.
 */
#define SQL_BEST_ROWID 				1
#define SQL_ROWVER 				2

#define SQL_SCOPE_CURROW 			0
#define SQL_SCOPE_TRANSACTION 			1
#define SQL_SCOPE_SESSION 			2


#if (ODBCVER >= 0x0300)
/* 
 * Bitmasks for SQL_DYNAMIC_CURSOR_ATTRIBUTES1,  
 * SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1, SQL_KEYSET_CURSOR_ATTRIBUTES1, 
 * and SQL_STATIC_CURSOR_ATTRIBUTES1.
 */

/*
 * SQLFetchScroll FetchOrientations 
 */
#define SQL_CA1_NEXT		0x00000001L
#define SQL_CA1_ABSOLUTE	0x00000002L
#define SQL_CA1_RELATIVE	0x00000004L
#define SQL_CA1_BOOKMARK	0x00000008L

/* 
 * SQLSetPos LockTypes 
 */
#define SQL_CA1_LOCK_NO_CHANGE	0x00000040L
#define SQL_CA1_LOCK_EXCLUSIVE	0x00000080L
#define SQL_CA1_LOCK_UNLOCK	0x00000100L

/* 
 * SQLSetPos Operations 
 */
#define SQL_CA1_POS_POSITION	0x00000200L
#define SQL_CA1_POS_UPDATE	0x00000400L
#define SQL_CA1_POS_DELETE	0x00000800L
#define SQL_CA1_POS_REFRESH	0x00001000L

/* 
 * Positioned updates and deletes 
 */
#define SQL_CA1_POSITIONED_UPDATE	0x00002000L
#define SQL_CA1_POSITIONED_DELETE	0x00004000L
#define SQL_CA1_SELECT_FOR_UPDATE	0x00008000L

/* 
 * SQLBulkOperations operations 
 */
#define SQL_CA1_BULK_ADD			0x00010000L
#define SQL_CA1_BULK_UPDATE_BY_BOOKMARK		0x00020000L
#define SQL_CA1_BULK_DELETE_BY_BOOKMARK		0x00040000L
#define SQL_CA1_BULK_FETCH_BY_BOOKMARK		0x00080000L

/* 
 * Bitmasks for SQL_DYNAMIC_CURSOR_ATTRIBUTES2,
 * SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2, SQL_KEYSET_CURSOR_ATTRIBUTES2, 
 * and SQL_STATIC_CURSOR_ATTRIBUTES2 
 */

/* 
 * Values for SQL_ATTR_SCROLL_CONCURRENCY 
 */
#define SQL_CA2_READ_ONLY_CONCURRENCY		0x00000001L
#define SQL_CA2_LOCK_CONCURRENCY		0x00000002L
#define SQL_CA2_OPT_ROWVER_CONCURRENCY		0x00000004L
#define SQL_CA2_OPT_VALUES_CONCURRENCY		0x00000008L

/* 
 * Sensitivity to inserts, deletes, and updates 
 */
#define SQL_CA2_SENSITIVITY_ADDITIONS		0x00000010L
#define SQL_CA2_SENSITIVITY_DELETIONS		0x00000020L
#define SQL_CA2_SENSITIVITY_UPDATES		0x00000040L

/* 
 * Semantics of SQL_ATTR_MAX_ROWS 
 */
#define SQL_CA2_MAX_ROWS_SELECT		0x00000080L
#define SQL_CA2_MAX_ROWS_INSERT		0x00000100L
#define SQL_CA2_MAX_ROWS_DELETE		0x00000200L
#define SQL_CA2_MAX_ROWS_UPDATE		0x00000400L
#define SQL_CA2_MAX_ROWS_CATALOG	0x00000800L
#define SQL_CA2_MAX_ROWS_AFFECTS_ALL	(SQL_CA2_MAX_ROWS_SELECT | SQL_CA2_MAX_ROWS_INSERT | SQL_CA2_MAX_ROWS_DELETE | SQL_CA2_MAX_ROWS_UPDATE | SQL_CA2_MAX_ROWS_CATALOG)


/* 
 * Semantics of SQL_DIAG_CURSOR_ROW_COUNT 
 */
#define SQL_CA2_CRC_EXACT	  0x00001000L
#define SQL_CA2_CRC_APPROXIMATE	  0x00002000L

/* 
 * Kinds of positioned statements that can be simulated 
 */
#define SQL_CA2_SIMULATE_NON_UNIQUE	0x00004000L
#define SQL_CA2_SIMULATE_TRY_UNIQUE	0x00008000L
#define SQL_CA2_SIMULATE_UNIQUE		0x00010000L
#endif  /* ODBCVER >= 0x0300 */

#endif /* _UDBCODBC_H */
